	SUBROUTINE BROYFA(OVERCH,OVERFL,SCLFCH,SCLXCH,MAXEXP,
     $                    N     ,NUNIT ,OUTPUT,EPSMCH,A     ,
     $                    DELF  ,FVEC  ,FVECC ,JAC   ,RDIAG ,
     $                    S     ,SCALEF,SCALEX,T     ,W     ,
     $                    XC    ,XPLUS )
C
C       FEB. 23, 1992
C
C       THE BROYDEN QUASI-NEWTON METHOD IS APPLIED TO THE FACTORED
C       FORM OF THE JACOBIAN.
C
C       NOTE: T AND W ARE TEMPORARY WORKING VECTORS ONLY.
C
C       THE UPDATE OCCURS ONLY IF A SIGNIFICANT CHANGE IN THE
C       JACOBIAN WOULD RESULT, I.E., NOT ALL THE VALUES IN VECTOR W
C       ARE LESS THAN THE THRESHOLD IN MAGNITUDE.  IF THE VECTOR
C       W IS ESSENTIALLY ZERO THEN THE LOGICAL VARIABLE SKIPUP
C       REMAINS SET AT TRUE.
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DOUBLE PRECISION JAC(N,N)
        INTEGER          OUTPUT
        DIMENSION        A(N,N)   ,DELF(N) ,FVEC(N)   ,FVECC(N) ,
     $                   RDIAG(N) ,S(N)    ,SCALEF(N) ,SCALEX(N),
     $                   T(N)     ,W(N)    ,XC(N)     ,XPLUS(N)
	LOGICAL          OVERCH   ,OVERFL  ,SCLFCH    ,SCLXCH   ,
     $                   SKIPUP
        DATA ZERO,TEN /0.0D0,10.0D0/
C
        OVERFL=.FALSE.
        EPS=TEN**(-MAXEXP)
        SQRTEP=SQRT(EPSMCH)
C
        DO 100 I=1,N
           A(I,I)=RDIAG(I)
           S(I)=XPLUS(I)-XC(I)
100     CONTINUE
C
C       R IS NOW IN THE UPPER TRIANGLE OF A.
C
        SKIPUP=.TRUE.
C
C       THE BROYDEN UPDATE IS CONDENSED INTO THE FORM
C
C       A(NEW) = A(OLD) + T S^
C
C       THE PRODUCT A*S IS FORMED IN TWO STAGES AS R IS IN THE UPPER
C       TRIANGLE OF MATRIX A AND Q^ IS IN JAC.
C
C       FIRST MULTIPLY R*S (A IS CONSIDERED UPPER TRIANGULAR)
C
        CALL UVMUL(N,N,N,N,A,S,T)
C
C       NOTE: THIS T IS TEMPORARY - IT IS THE T FROM BELOW WHICH
C             IS SENT TO SUBROUTINE QRUPDA.
C
        DO 200 I=1,N
	   CALL INNERP(OVERCH,OVERFL,MAXEXP,N     ,N     ,N     ,
     $                 NUNIT ,OUTPUT,SUM   ,JAC(N*(I-1)+1,1),T)
           W(I)=SCALEF(I)*(FVEC(I)-FVECC(I))-SUM
C
C          TEST TO ENSURE VECTOR W IS NONZERO.  ANY VALUE GREATER
C          THAN THE THRESHOLD WILL SET SKIPUP TO FALSE.
C
           IF(ABS(W(I)).GT.SQRTEP*SCALEF(I)*(ABS(FVEC(I))+
     $     ABS(FVECC(I)))) THEN
              SKIPUP=.FALSE.
           ELSE
              W(I)=ZERO
           END IF
200     CONTINUE
C
C       IF W(I)=0 FOR ALL I THEN THE UPDATE IS SKIPPED.
C
        IF(.NOT.SKIPUP) THEN
C
C          T=Q^W; Q^ IS IN JAC.
C
	   CALL AVMUL(N,N,N,N,JAC,W,T)
	   IF(SCLXCH) THEN
	      DO 300 I=1,N
		 W(I)=S(I)*SCALEX(I)
300           CONTINUE
	   ELSE
	      CALL MATCOP(N,N,1,1,N,1,S,W)
	   END IF
           CALL TWONRM(OVERFL,MAXEXP,N,EPSMCH,DENOM,W)
C
C          IF OVERFLOW WOULD OCCUR MAKE NO CHANGE TO JACOBIAN.
C
           IF(OVERFL.OR.LOG10(DENOM+EPS).GT.MAXEXP/2) THEN
              IF(OUTPUT.GT.3) THEN
                 WRITE(NUNIT,1)
1                FORMAT(T3,'*',T74,'*')
                 WRITE(NUNIT,2)
2                FORMAT(T3,'*',4X,'WARNING: JACOBIAN NOT UPDATED',
     $           ' TO AVOID OVERFLOW IN DENOMINATOR OF',T74,'*')
                 WRITE(NUNIT,3)
3                FORMAT(T3,'*',4X,'BROYDEN UPDATE',T74,'*')
              END IF
              RETURN
           ELSE
	      DENOM=DENOM*DENOM
           END IF
C
C          IF DENOM IS ZERO AVOID DIVIDE BY ZERO AND CONTINUE WITH
C          SAME JACOBIAN.
C
           IF(DENOM.EQ.ZERO) RETURN
C
C          THE SCALED VERSION OF S REPLACES THE ORIGINAL BEFORE
C          BEING SENT TO QRUPDA.
C
           DO 400 I=1,N
	      S(I)=S(I)*SCALEX(I)*SCALEX(I)/DENOM
400        CONTINUE
C
C          UPDATE THE QR DECOMPOSITION USING A SERIES OF GIVENS
C          ROTATIONS.
C
           CALL QRUPDA(OVERFL,MAXEXP,N,EPSMCH,A,JAC,T,S)
C
C          RESET RDIAG AS DIAGONAL OF CURRENT R WHICH IS IN
C          THE UPPER TRIANGE OF A.
C
           DO 500 I=1,N
              RDIAG(I)=A(I,I)
500        CONTINUE
        END IF
C
C       UPDATE THE GRADIENT VECTOR, DELF.  THE NEW Q^ IS IN JAC.
C
C       DELF = (QR)^F = R^Q^F = R^JAC F
C
	IF(SCLFCH) THEN
	   DO 600 I=1,N
	      W(I)=FVEC(I)*SCALEF(I)
600        CONTINUE
	ELSE
	   CALL MATCOP(N,N,1,1,N,1,FVEC,W)
	END IF
        CALL AVMUL(N,N,N,N,JAC,W,T)
        CALL UTBMUL(N,N,1,1,N,N,A,T,DELF)
        RETURN
C
C       LAST CARD OF SUBROUTINE BROYFA.
C
        END
