        SUBROUTINE CONDNO(OVERCH,OVERFL,MAXEXP,N     ,NUNIT ,
     $                    OUTPUT,CONNUM,A     ,P     ,PM    ,
     $                    Q     ,RDIAG )
C
C       FEB. 14, 1991
C
C       THIS SUBROUTINE ESTIMATES THE CONDITION NUMBER OF A
C       QR-DECOMPOSED MATRIX USING THE METHOD OF CLINE, MOLER,
C       STEWART AND WILKINSON (SIAM J. N.A. 16 P368 (1979) ).
C
C       IF A POTENTIAL OVERFLOW IS DETECTED AT ANY POINT THEN A
C       CONDITION NUMBER EQUIVALENT TO THAT OF A SINGULAR MATRIX
C       IS ASSIGNED BY THE CALLING SUBROUTINE.
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DIMENSION     A(N,N) ,P(N)  ,PM(N)  ,RDIAG(N)  ,Q(N)
        LOGICAL       OVERCH ,OVERFL
        DATA ZERO,ONE,TEN /0.0D0,1.0D0,10.0D0/
C
        OVERFL=.FALSE.
        EPS=TEN**(-MAXEXP)
C
        CONNUM=ABS(RDIAG(1))
        DO 100 J=2,N
           TEMP=ZERO
           DO 200 I=1,J-1
              IF(OVERCH) THEN
                 IF(ABS(A(I,J)).GT.TEN**(MAXEXP-1)) THEN
                    OVERFL=.TRUE.
                    RETURN
                 END IF
              END IF
              TEMP=TEMP+ABS(A(I,J))
200        CONTINUE
           TEMP=TEMP+ABS(RDIAG(J))
           CONNUM=MAX(CONNUM,TEMP)
100     CONTINUE
        Q(1)=ONE/RDIAG(1)
        DO 300 I=2,N
           IF(OVERCH) THEN
              IF(LOG10(ABS(Q(1))+EPS)+LOG10(ABS(A(1,I))+EPS)
     $           .GT.MAXEXP) THEN
                 OVERFL=.TRUE.
                 RETURN
              END IF
           END IF
           P(I)=A(1,I)*Q(1)
300     CONTINUE
        DO 400 J=2,N
           IF(OVERCH) THEN
              IF(LOG10(ABS(P(J))+EPS)-LOG10(ABS(RDIAG(J))+EPS)
     $           .GT.MAXEXP) THEN
                 OVERFL=.TRUE.
                 RETURN
              END IF
           END IF
           QP=(ONE-P(J))/RDIAG(J)
           QM=(-ONE-P(J))/RDIAG(J)
           TEMP=ABS(QP)
           TEMPM=ABS(QM)
           DO 500 I=J+1,N
              IF(OVERCH) THEN
                 IF(LOG10(ABS(A(J,I))+EPS)+LOG10(ABS(QM)+EPS)
     $              .GT.MAXEXP) THEN
                    OVERFL=.TRUE.
                    RETURN
                 END IF
              END IF
              PM(I)=P(I)+A(J,I)*QM
              IF(OVERCH) THEN
                 IF(LOG10(ABS(PM(I))+EPS)-LOG10(ABS(RDIAG(I))+EPS)
     $              .GT.MAXEXP) THEN
                    OVERFL=.TRUE.
                    RETURN
                 END IF
              END IF
              TEMPM=TEMPM+(ABS(PM(I))/ABS(RDIAG(I)))
              IF(OVERCH) THEN
                 IF(TEMPM.GT.TEN**(MAXEXP-1)) THEN
                    OVERFL=.TRUE.
                    RETURN
                 END IF
              END IF
              IF(OVERCH) THEN
                 IF(LOG10(ABS(A(J,I))+EPS)+LOG10(ABS(QP)+EPS)
     $              .GT.MAXEXP) THEN
                    OVERFL=.TRUE.
                    RETURN
                 END IF
              END IF
              P(I)=P(I)+A(J,I)*QP
              IF(OVERCH) THEN
                 IF(LOG10(ABS(P(I))+EPS)-LOG10(ABS(RDIAG(I))+EPS)
     $              .GT.MAXEXP) THEN
                    OVERFL=.TRUE.
                    RETURN
                 END IF
              END IF
              TEMP=TEMP+(ABS(P(I))/ABS(RDIAG(I)))
              IF(OVERCH) THEN
                 IF(TEMP.GT.TEN**(MAXEXP-1)) THEN
                    OVERFL=.TRUE.
                    RETURN
                 END IF
              END IF
500        CONTINUE
           IF(TEMP.GE.TEMPM) THEN
              Q(J)=QP
           ELSE
              Q(J)=QM
              DO 600 I=J+1,N
                 P(I)=PM(I)
600           CONTINUE
           END IF
400     CONTINUE
        QNORM=ZERO
        DO 700 J=1,N
           QNORM=QNORM+ABS(Q(J))
           IF(OVERCH) THEN
              IF(QNORM.GT.TEN**(MAXEXP-1)) THEN
                 OVERFL=.TRUE.
                 RETURN
              END IF
           END IF
700     CONTINUE
        IF(LOG10(CONNUM)-LOG10(QNORM).GT.MAXEXP) THEN
           OVERFL=.TRUE.
           RETURN
        END IF
        CONNUM=CONNUM/QNORM
        CALL RSOLV(OVERCH,OVERFL,MAXEXP,N,NUNIT,OUTPUT,A,RDIAG,Q)
        IF(OVERFL) RETURN
        QNORM=ZERO
        DO 800 J=1,N
           QNORM=QNORM+ABS(Q(J))
           IF(OVERCH) THEN
              IF(QNORM.GT.TEN**(MAXEXP-1)) THEN
                 OVERFL=.TRUE.
                 RETURN
              END IF
           END IF
800     CONTINUE
        CONNUM=CONNUM*QNORM
        RETURN
C
C       LAST CARD OF SUBROUTINE CONDNO.
C
        END
