        SUBROUTINE DELCAU(CAUCHY,OVERCH,OVERFL,ITNUM ,MAXEXP,
     $                    N     ,NUNIT ,OUTPUT,BETA  ,CAULEN,
     $                    DELTA ,EPSMCH,MAXSTP,NEWLEN,SQRTZ ,
     $                    A     ,DELF  ,SCALEX,WV1   )
C
C       FEB. 23, 1992
C
C       THIS SUBROUTINE ESTABLISHES AN INITIAL TRUST REGION, DELTA,
C       IF ONE IS NOT SPECIFIED BY THE USER AND FINDS THE LENGTH OF
C       THE SCALED CAUCHY STEP, CAULEN, AT EACH STEP IF THE DOUBLE
C       DOGLEG OPTION IS BEING USED.
C
C       THE USER HAS TWO CHOICES FOR THE INITIAL TRUST REGION:
C          1)  BASED ON THE SCALED CAUCHY STEP
C          2)  BASED ON THE SCALED NEWTON STEP
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	DOUBLE PRECISION  MAXSTP   ,NEWLEN
        INTEGER           OUTPUT
        DIMENSION         A(N,N)   ,DELF(N) ,SCALEX(N) ,WV1(N)
        LOGICAL           CAUCHY   ,OVERCH  ,OVERFL
	DATA ZERO,THREE,TEN/0.0D0,3.0D0,10.0D0/
C
        OVERFL=.FALSE.
	EPS=TEN**(-MAXEXP)
C
C       IF DELTA IS NOT GIVEN EVALUATE IT USING EITHER THE CAUCHY
C       STEP OR THE NEWTON STEP AS SPECIFIED BY THE USER.
C
C       THE SCALED CAUCHY LENGTH, CAULEN, IS REQUIRED IN TWO CASES.
C       1) WHEN SELECTED AS THE CRITERION FOR THE INITIAL DELTA
C       2) IN THE DOUBLE DOGLEG STEP REGARDLESS OF (1)
C
       IF(OUTPUT.GT.3) THEN
           WRITE(NUNIT,1)
1          FORMAT(T3,'*',T74,'*')
           WRITE(NUNIT,1)
           WRITE(NUNIT,2)
2          FORMAT(T3,'*',4X,'DETERMINATION OF SCALED CAUCHY',
     $     ' STEP LENGTH, CAULEN',T74,'*')
	END IF
C
C       FIND FACTOR WHICH GIVES CAUCHY POINT WHEN MULTPLYING
C       STEEPEST DESCENT DIRECTION, DELF.
C
C       CAULEN= ZETA**1.5/BETA
C             =  SQRTZ**3/BETA
C
        DO 100 I=1,N
           WV1(I)=DELF(I)/SCALEX(I)
100     CONTINUE
	CALL TWONRM(OVERFL,MAXEXP,N,EPSMCH,SQRTZ,WV1)
        IF(OVERFL) THEN
	   CAULEN=TEN**MAXEXP
           IF(OUTPUT.GT.4) THEN
              WRITE(NUNIT,1)
              WRITE(NUNIT,3) SQRTZ
3             FORMAT(T3,'*',7X,'ZETA SET TO ',1PD11.3,' TO'
     $        ' AVOID OVERFLOW',T74,'*')
              WRITE(NUNIT,4) CAULEN
4             FORMAT(T3,'*',7X,'SCALED CAUCHY LENGTH, CAULEN ',
     $        'SET TO',1PD9.2,' TO AVOID OVERFLOW',T74,'*')
              IF(ITNUM.EQ.1) THEN
                 WRITE(NUNIT,5)
5                FORMAT(T3,'*',7X,'THE PROBLEM SHOULD BE RESCALED',
     $           ' OR A NEW STARTING POINT CHOSEN',T74,'*')
                 WRITE(NUNIT,6)
6                FORMAT(T3,'*',7X,'EXECUTION CONTINUES WITH',
     $           ' SUBSTITUTIONS AS LISTED ABOVE',T74,'*')
              END IF
           END IF
        ELSE
           IF(OUTPUT.GT.4) THEN
              WRITE(NUNIT,1)
              WRITE(NUNIT,7) SQRTZ
7             FORMAT(T3,'*',7X,'SQUARE ROOT OF ZETA, SQRTZ: ',
     $        1PD12.3,T74,'*')
           END IF
        END IF
C
C       NOTE: THE LOWER TRIANGLE OF MATRIX A NOW CONTAINS THE
C             TRANSPOSE OF R WHERE A=QR.
C
	BETA=ZERO
        DO 200 I=1,N
	   TEMP=ZERO
           DO 300 J=I,N
              IF(OVERCH) THEN
		 IF(LOG10(ABS(A(J,I))+EPS)+LOG10(ABS(DELF(J))+EPS).
     $              GT.MAXEXP) THEN
		    CAULEN=SQRT(EPSMCH)
                    GO TO 301
                 END IF
              END IF
	      TEMP=TEMP+A(J,I)*DELF(J)/(SCALEX(J)*SCALEX(J))
300        CONTINUE
           BETA=BETA+TEMP*TEMP
200     CONTINUE
        IF(OUTPUT.GT.4) THEN
           WRITE(NUNIT,1)
           WRITE(NUNIT,8) BETA
8          FORMAT(T3,'*',7X,'BETA: ',1PD11.3,6X,'NOTE: ',
     $     'CAULEN=ZETA**1.5/BETA',T74,'*')
           WRITE(NUNIT,1)
        END IF
C
C       AVOID OVERFLOWS IN FINDING CAULEN.
C
	IF(THREE*LOG10(SQRTZ+EPS)-LOG10(BETA+EPS).LT.MAXEXP
     $  .AND.(.NOT.OVERFL).AND.BETA.NE.ZERO) THEN
C
C          NORMAL DETERMINATION.
C
	   CAULEN=SQRTZ*SQRTZ*SQRTZ/BETA
C
C          THIS STEP AVOIDS DIVIDE BY ZERO IN DOGLEG IN THE
C          (RARE) CASE WHERE DELF(I)=0 FOR ALL I BUT THE
C          POINT IS NOT YET A SOLUTION - MOST LIKELY A BAD
C          STARTING ESTIMATE.
C
	   CAULEN=MAX(CAULEN,TEN**(-MAXEXP))
C
        ELSE
C
C          SUBSTITUTION TO AVOID OVERFLOW.
C
	   CAULEN=TEN**MAXEXP
        END IF
301     CONTINUE
        IF(OUTPUT.GT.3) THEN
           WRITE(NUNIT,1)
           WRITE(NUNIT,9) CAULEN
9          FORMAT(T3,'*',7X,'SCALED CAUCHY LENGTH, CAULEN: ',
     $     1PD12.3,T74,'*')
        END IF
C
C       ESTABLISH INITIAL TRUST REGION IF NEEDED.
C
	IF(DELTA.LT.ZERO) THEN
C
C          USE DISTANCE TO CAUCHY POINT OR LENGTH OF NEWTON STEP.
C
           IF(CAUCHY) THEN
              DELTA=MIN(CAULEN,MAXSTP)
           ELSE
              DELTA=MIN(NEWLEN,MAXSTP)
           END IF
           IF(OUTPUT.GT.3) THEN
              WRITE(NUNIT,1)
              WRITE(NUNIT,10) DELTA
10            FORMAT(T3,'*',7X,'INITIAL TRUST REGION SIZE, DELTA: ',
     $        1PD12.3,T74,'*')
           END IF
        END IF
        RETURN
C
C       LAST CARD OF SUBROUTINE DELCAU.
C
	END
