        SUBROUTINE DEUFLS(ABORT ,DEUFLH,GEOMS ,OVERCH,OVERFL,
     $                    QNFAIL,QRSING,RESTRT,SCLFCH,SCLXCH,
     $                    ACPCOD,ACPTCR,CONTYP,ITNUM ,JUPDM ,
     $                    MAXEXP,MAXLIN,N     ,NFUNC ,NUNIT ,
     $                    OUTPUT,QNUPDM,STOPCR,ALPHA ,CONFAC,
     $                    DELFTS,EPSMCH,FCNMAX,FCNNEW,FCNOLD,
     $                    LAMBDA,NEWMAX,SBRNRM,SIGMA ,A     ,
     $                    ASTORE,BOUNDL,BOUNDU,DELF  ,FVEC  ,
     $                    HHPI  ,JAC   ,RDIAG ,RHS   ,S     ,
     $                    SBAR  ,SCALEF,SCALEX,SN    ,WV2   ,
     $                    XC    ,XPLUS ,FVECEV)
C
C       FEB. 23, 1992
C
C       THIS SUBROUTINE CONDUCTS A LINE SEARCH IN THE NEWTON
C       DIRECTION IF NO CONSTRAINTS ARE VIOLATED.  IF THE FIRST
C       TRIAL IS A FAILURE THERE ARE TWO TYPES OF LINE SEARCH
C       AVAILABLE.
C         1)  REDUCE THE RELAXATION FACTOR, LAMBDA, TO
C             SIGMA*LAMBDA WHERE SIGMA IS USER-SPECIFIED
C             (GEOMETRIC LINE SEARCH)
C         2)  AT THE FIRST STEP MINIMIZE A QUADRATIC THROUGH
C             THE OBJECTIVE FUNCTION AT THE CURRENT POINT AND
C             TRIAL ESTIMATE (WHICH MUST BE A FAILURE) WITH
C             INITIAL SLOPE DELFTS.  AT SUBSEQUENT STEPS MINI-
C             MIZE A CUBIC THROUGH THE OBJECTIVE FUNCTION AT
C             THE TWO MOST RECENT FAILURES AND THE CURRENT
C             POINT, AGAIN USING THE INITIAL SLOPE, DELFTS.
C
C       CONVIO  INDICATES A CONSTRAINT VIOLATION BY ONE OR MORE
C               COMPONENTS
C       FRSTST  INDICATES THE FIRST STEP IN THE LINE SEARCH.
C
C       RATIO   RATIO OF PROPOSED STEP LENGTH IN (I)TH DIRECTION
C               TO DISTANCE FROM (I)TH COMPONENT TO BOUNDARY
C               VIOLATED
C       RATIOM  MINIMUM OF RATIOS FOR ALL CONSTAINTS VIOLATED
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DOUBLE PRECISION JAC(N,N) ,LAMBDA     ,LAMPRE   ,LAMTMP   ,
     $                   NEWMAX
        INTEGER          ACPCOD   ,ACPTCR     ,CONTYP   ,OUTPUT   ,
     $                   STOPCR   ,QNUPDM
        DIMENSION        A(N,N)   ,ASTORE(N,N),BOUNDL(N),BOUNDU(N),
     $                   FVEC(N)  ,HHPI(N)    ,RDIAG(N) ,RHS(N)   ,
     $                   S(N)     ,DELF(N)    ,SBAR(N)  ,SCALEF(N),
     $                   SCALEX(N),SN(N)      ,WV2(N)   ,XC(N)    ,
     $                   XPLUS(N)
        LOGICAL          ABORT    ,CONVIO     ,DEUFLH   ,FRSTST   ,
     $                   GEOMS    ,OVERCH     ,OVERFL   ,QNFAIL   ,
     $                   QRSING   ,RESTRT     ,SCLFCH   ,SCLXCH   ,
     $                   WRNSUP
        COMMON/NNES_2/WRNSUP
        EXTERNAL FVECEV
        DATA ZERO,POINT1,POINT5,ONE,TWO,THREE,TEN /0.0D0,0.1D0,0.5D0,
     $  1.0D0,2.0D0,3.0D0,10.0D0/
C
	FRSTST=.TRUE.
        OVERFL=.FALSE.
        EPS=TEN**(-MAXEXP)
        DO 100 K=1,MAXLIN
           RATIOM=ONE
C
           CONVIO=.FALSE.
C
C          FIND TRIAL POINT AND CHECK IF CONSTRAINTS VIOLATED (IF
C          CONTYP IS NOT EQUAL TO ZERO).
C
           DO 200 I=1,N
C
C             NOTE: WV2 MARKS VIOLATIONS.  WV2(I) CHANGES TO
C                   1 FOR LOWER BOUND VIOLATIONS AND TO 2 FOR
C                   FOR UPPER BOUND VIOLATIONS.  CONSTRAINT VIOL-
C                   ATIONS CAN ONLY OCCUR AT THE FIRST STEP.
C
              WV2(I)=-ONE
              XPLUS(I)=XC(I)+LAMBDA*SN(I)
              IF(CONTYP.GT.0.AND.FRSTST) THEN
                 IF(XPLUS(I).LT.BOUNDL(I)) THEN
                    CONVIO=.TRUE.
                    WV2(I)=ONE
                 ELSEIF(XPLUS(I).GT.BOUNDU(I)) THEN
                    CONVIO=.TRUE.
                    WV2(I)=TWO
                 ELSE
                    WV2(I)=-ONE
                 END IF
              END IF
200        CONTINUE
C
C          IF CONSTRAINTS ARE VIOLATED FIRST REDUCE THE STEP
C          SIZES FOR THE VIOLATING COMPONENTS TO OBTAIN A
C          FEASIBLE POINT.  IF THE DIRECTION TO THIS MODIFIED
C          POINT IS NOT A DESCENT DIRECTION OR IF THE MODIFIED
C          STEP DOES NOT LEAD TO AN ACCEPTABLE POINT THEN RETURN
C          TO THE NEWTON DIRECTION AND START A LINE SEARCH AT A
C          FEASIBLE POINT WHERE THE COMPONENT WHICH HAS THE
C          SMALLEST VALUE OF RATIO (DEFINED BELOW) IS TAKEN TO
C          "CONFAC" OF THE DISTANCE TO THE BOUNDARY.  DEFAULT
C          VALUE OF CONFAC IS 0.95.
C
           IF(CONVIO) THEN
              IF(OUTPUT.GT.3) THEN
                 WRITE(NUNIT,1)
1                FORMAT(T3,'*',T74,'*')
		 WRITE(NUNIT,2) K
2                FORMAT(T3,'*',10X,'LINE SEARCH STEP:',I3,T74,'*')
		 WRITE(NUNIT,1)
		 WRITE(NUNIT,3) LAMBDA
3                FORMAT(T3,'*',10X,'LAMBDA FOR ATTEMPTED STEP: ',
     $           1PD12.3,T74,'*')
		 WRITE(NUNIT,4)
4                FORMAT(T3,'*',10X,'CONSTRAINT VIOLATED',T74,'*')
                 WRITE(NUNIT,1)
                 WRITE(NUNIT,5)
5                FORMAT(T3,'*',10X,'TRIAL ESTIMATES (VIOLATIONS',
     $           ' MARKED)',T74,'*')
                 WRITE(NUNIT,1)
                 DO 300 I=1,N
                    IF(WV2(I).GT.ZERO) THEN
                       WRITE(NUNIT,6) I,XPLUS(I)
6                      FORMAT(T3,'*',13X,'XPLUS(',I3,') = ',1PD12.3,
     $                 2X,'*',T74,'*')
                    ELSE
                       WRITE(NUNIT,7) I,XPLUS(I)
7                      FORMAT(T3,'*',13X,'XPLUS(',I3,') = ',1PD12.3,
     $                 T74,'*')
                    END IF
300              CONTINUE
              END IF
              DO 400 I=1,N
                 IF(WV2(I).GT.ZERO) THEN
C
C                   FIND RATIO FOR THIS VIOLATING COMPONENT.
C
                    IF(WV2(I).EQ.ONE) THEN
                       RATIO=-(XC(I)-BOUNDL(I))/
     $                        (XPLUS(I)-XC(I))
                    ELSEIF(WV2(I).EQ.TWO) THEN
                       RATIO=(BOUNDU(I)-XC(I))/
     $                       (XPLUS(I)-XC(I))
                    END IF
C
C                   NOTE: THIS LINE IS FOR OUTPUT PURPOSES ONLY.
C
                    WV2(I)=RATIO
C
                    RATIOM=MIN(RATIOM,RATIO)
                    IF(RATIO.GT.POINT5) THEN
                       S(I)=CONFAC*RATIO*LAMBDA*SN(I)
                    ELSE
C
C                      WITHIN BUFFER ZONE - ONLY TAKE 1/2
C                      OF THE STEP YOU WOULD TAKE OTHERWISE.
C
                       S(I)=POINT5*CONFAC*RATIO*LAMBDA*SN(I)
                    END IF
C
C                   ESTABLISH MODIFIED TRIAL POINT.
C
                    XPLUS(I)=XC(I)+S(I)
                 ELSE
C
C                   FOR NONVIOLATORS XPLUS REMAINS UNCHANGED BUT
C                   THE COMPONENT OF S IS LOADED TO CHECK THE
C                   DIRECTIONAL DERIVATIVE.
C
                    S(I)=LAMBDA*SN(I)
                 END IF
400           CONTINUE
              IF(OUTPUT.GT.3) THEN
                 WRITE(NUNIT,1)
                 WRITE(NUNIT,8)
8                FORMAT(T3,'*',7X,'NEW S AND XPLUS VECTORS',
     $           ' (WITH RATIOS FOR VIOLATIONS)',T74,'*')
                 WRITE(NUNIT,1)
                 WRITE(NUNIT,9)
9                FORMAT(T3,'*',7X,'NOTE: RATIOS ARE RATIO OF',
     $           ' LENGTH TO BOUNDARY FROM CURRENT',T74,'*')
                 WRITE(NUNIT,10)
10               FORMAT(T3,'*',13X,'X VECTOR TO MAGNITUDE OF',
     $           ' CORRESPONDING PROPOSED STEP',T74,'*')
                 WRITE(NUNIT,1)
                 DO 500 I=1,N
                    IF(WV2(I).LT.ZERO) THEN
                       WRITE(NUNIT,11) I,S(I),I,XPLUS(I)
11                     FORMAT(T3,'*',7X,'S(',I3,') = ',1PD12.3,4X,
     $                 'XPLUS(',I3,') = ',1PD12.3,T74,'*')
                    ELSE
                       WRITE(NUNIT,12) I,S(I),I,XPLUS(I),WV2(I)
12                     FORMAT(T3,'*',7X,'S(',I3,') = ',1PD12.3,4X,
     $                 'XPLUS(',I3,') = ',1PD12.3,1X,1PD11.3,T74,'*')
                    END IF
500              CONTINUE
                 WRITE(NUNIT,1)
                 WRITE(NUNIT,13) RATIOM
13               FORMAT(T3,'*',7X,'MINIMUM OF RATIOS, RATIOM: ',
     $           1PD12.3,T74,'*')
              END IF
C
C             CHECK DIRECTIONAL DERIVATIVE FOR MODIFIED POINT, DLFTSM.
C
	      CALL INNERP(OVERCH,OVERFL,MAXEXP,N     ,N     ,N     ,
     $                    NUNIT ,OUTPUT,DLFTSM,DELF  ,S     )
              OVERFL=.FALSE.
              IF(OUTPUT.GT.3) THEN
                 WRITE(NUNIT,1)
                 WRITE(NUNIT,14) DLFTSM
14               FORMAT(T3,'*',7X,'INNER PRODUCT OF DELF AND S FOR',
     $           ' MODIFIED S: ',1PD12.3,T74,'*')
              END IF
C
C             IF INNER PRODUCT IS POSITIVE RETURN TO NEWTON DIRECTION
C             AND CONDUCT A LINE SEARCH WITHIN THE FEASIBLE REGION.
C
              IF(DLFTSM.GT.ZERO) THEN
                 IF(OUTPUT.GT.3) THEN
                    WRITE(NUNIT,15)
15                  FORMAT(T3,'*',7X,'DELFTS > 0','  START LINE',
     $              ' SEARCH AT LAMBDA=CONFAC*LAMBDA*RATIOM',T74,'*')
                    WRITE(NUNIT,16)
16                  FORMAT(T3,'*',7X,'NOTE: NO TRIAL POINT WAS',
     $              ' EVALUATED AT THIS STEP OF LINE SEARCH',T74,'*')
                 END IF
C
C                THE STARTING POINT IS SET AT JUST INSIDE THE MOST
C                VIOLATED BOUNDARY.
C
                 LAMBDA=CONFAC*RATIOM*LAMBDA
C
C                LAMBDA IS ALREADY SET - SKIP NORMAL PROCEDURE.
C
                 GO TO 101
              END IF
C
           END IF
C
C          NO CONSTRAINTS VIOLATED - EVALUATE RESIDUAL VECTOR
C          AT NEW POINT.
C
           CALL FVECEV(OVERFL,N,FVEC,XPLUS)
           NFUNC=NFUNC+1
C
C          CHECK FOR OVERFLOW IN FUNCTION VECTOR EVALUATION.
C          IF SO, REDUCE STEP LENGTH AND CONTINUE LINE SEARCH.
C
           IF(OVERFL) THEN
C
              IF(OUTPUT.GT.3) THEN
                 WRITE(NUNIT,17)
17               FORMAT(T3,'*',7X,'OVERFLOW IN FUNCTION VECTOR',
     $           ' - STEP LENGTH REDUCED',T74,'*')
C
C                FORCE STEP TO BE WITHIN CONSTRAINTS - DON'T CALL
C                THIS THE FIRST STEP, I.E. FRSTST STAYS AT TRUE.
C
              END IF
              IF(CONVIO) THEN
                 LAMBDA=RATIOM*CONFAC*LAMBDA
              ELSE
                 LAMBDA=SIGMA*LAMBDA
              END IF
C
C             LAMBDA IS ALREADY SET - SKIP NORMAL PROCEDURE.
C
              GO TO 101
           END IF
C
C          EVALUATE (POSSIBLY SCALED) OBJECTIVE FUNCTION AT NEW POINT.
C
           CALL FCNEVL(OVERFL,MAXEXP,N     ,NUNIT ,OUTPUT,
     $                 EPSMCH,FCNNEW,FVEC  ,SCALEF,WV2   )
           IF(OVERFL) THEN
C
              IF(OUTPUT.GT.3) THEN
                 WRITE(NUNIT,18)
18               FORMAT(T3,'*',7X,'OVERFLOW IN OBJECTIVE FUNCTION',
     $           ' - STEP LENGTH REDUCED',T74,'*')
C
C                FORCE STEP TO BE WITHIN CONSTRAINTS - DON'T CALL
C                THIS THE FIRST STEP, I.E. FRSTST STAYS AT TRUE.
C
              END IF
              IF(CONVIO) THEN
                 LAMBDA=RATIOM*CONFAC*LAMBDA
              ELSE
                 LAMBDA=SIGMA*LAMBDA
              END IF
              GO TO 101
           END IF
C
C          IF DEUFLHARD'S METHOD IS BEING USED FOR EITHER
C          RELAXATION FACTOR INITIALIZATION OR THE SECOND
C          ACCEPTANCE CRITERION THEN EVALUATE SBAR.  EVALU-
C          ATION METHOD DEPENDS UPON WHETHER THE JACOBIAN
C          WAS PERTURBED IN THE SOLUTION OF THE LINEAR SYSTEM.
C          LOGICAL VARIABLE QRSING IS TRUE IF PERTURBATION
C          TOOK PLACE.
C
           IF(DEUFLH.OR.ACPTCR.EQ.12) THEN
	      IF(QRSING) THEN
C
C                FORM -J^F AS RIGHT HAND SIDE - METHOD DEPENDS ON
C                WHETHER QNUPDM EQUALS 0 OR 1 IF A QUASI-NEWTON
C                UPDATE IS BEING USED.  IF JUPDM IS 0 THEN THE NEWTON
C                STEP HAS BEEN FOUND IN SUBROUTINE NSTPUN.
C
		 IF(JUPDM.EQ.0.OR.QNUPDM.EQ.0) THEN
C
C                   UNSCALED JACOBIAN IN MATRIX JAC.
C
		    DO 600 I=1,N
		       IF(SCLFCH) THEN
			  WV2(I)=-FVEC(I)*SCALEF(I)*SCALEF(I)
		       ELSE
			  WV2(I)=-FVEC(I)
		       END IF
600                 CONTINUE
                    CALL ATBMUL(N,N,1,1,N,N,JAC,WV2,RHS)
                 ELSE
C
C                   R IN UPPER TRIANGLE OF A PLUS RDIAG AND Q^ IN JAC
C                   - FROM QR DECOMPOSITION OF SCALED JACOBIAN.
C
                    DO 700 I=1,N
                       WV2(I)=ZERO
                       DO 800 J=1,N
                          WV2(I)=WV2(I)-JAC(I,J)*FVEC(J)*SCALEF(J)
800                    CONTINUE
700                 CONTINUE
                    RHS(1)=RDIAG(1)*WV2(1)
                    DO 900 J=2,N
                       RHS(J)=ZERO
                       DO 1000 I=1,J-1
                          RHS(J)=RHS(J)+A(I,J)*WV2(I)
1000                   CONTINUE
                       RHS(J)=RHS(J)+RDIAG(J)*WV2(J)
900                 CONTINUE
                 END IF
                 CALL CHSOLV(OVERCH,OVERFL,MAXEXP,N     ,NUNIT ,
     $                       OUTPUT,A     ,RHS   ,SBAR  ,WV2   )
              ELSE
C
C                RIGHT HAND SIDE IS -FVEC.
C
                 IF(QNUPDM.EQ.0.OR.JUPDM.EQ.0) THEN
C
C                   QR DECOMPOSITION OF SCALED JACOBIAN STORED IN
C                   ASTORE.
C
                    DO 1100 I=1,N
                       SBAR(I)=-FVEC(I)*SCALEF(I)
1100                CONTINUE
                    CALL QRSOLV(OVERCH,OVERFL,MAXEXP,N     ,NUNIT ,
     $                          OUTPUT,ASTORE,HHPI  ,RDIAG ,SBAR  )
                 ELSE
C
C                   SET UP RIGHT HAND SIDE - MULTIPLY -FVEC BY Q^
C                   (STORED IN JAC).
C
                    DO 1200 I=1,N
                       WV2(I)=-FVEC(I)*SCALEF(I)
1200                CONTINUE
                    CALL AVMUL(N,N,N,N,JAC,WV2,SBAR)
                    CALL RSOLV(OVERCH,OVERFL,MAXEXP,N     ,NUNIT ,
     $                         OUTPUT,A     ,RDIAG ,SBAR  )
                 END IF
              END IF
C
C             NORM OF SCALED SBAR IS NEEDED FOR SECOND ACCEPTANCE TEST.
C
              IF(ACPTCR.EQ.12) THEN
		 DO 1300 I=1,N
		    WV2(I)=SCALEX(I)*SBAR(I)
1300             CONTINUE
		 CALL TWONRM(OVERFL,MAXEXP,N,EPSMCH,SBRNRM,WV2)
              END IF
           END IF
C
           IF(OUTPUT.GT.3) THEN
              WRITE(NUNIT,1)
	      WRITE(NUNIT,2) K
              WRITE(NUNIT,1)
	      WRITE(NUNIT,3) LAMBDA
              WRITE(NUNIT,1)
              IF(.NOT.CONVIO) THEN
                 WRITE(NUNIT,19)
19               FORMAT(T3,'*',10X,'NEW COMPONENT/FCN VECTORS',
     $           '  (XPLUS(I)=XC(I)+LAMBDA*SN(I))',T74,'*')
              ELSE
                 WRITE(NUNIT,20)
20               FORMAT(T3,'*',10X,'NEW FUNCTION VECTORS',
     $           ' AT MODIFIED POINT',T74,'*')
              END IF
              WRITE(NUNIT,1)
	      DO 1400 I=1,N
                 WRITE(NUNIT,21) I,XPLUS(I),I,FVEC(I)
21               FORMAT(T3,'*',10X,'XPLUS(',I3,') = ',1PD12.3,
     $           5X,'FVEC(',I3,') = ',1PD12.3,T74,'*')
1400          CONTINUE
	      WRITE(NUNIT,1)
	      IF(.NOT.SCLFCH) THEN
		 WRITE(NUNIT,22) FCNNEW
22               FORMAT(T3,'*',10X,'OBJECTIVE FUNCTION VALUE',
     $           ' AT XPLUS: .........'1PD12.3,T74,'*')
	      ELSE
		 WRITE(10,23) FCNNEW
23               FORMAT(T3,'*',10X,'SCALED OBJECTIVE FUNCTION VALUE',
     $           ' AT XPLUS: ..'1PD12.3,T74,'*')
	      END IF
	      WRITE(NUNIT,24) FCNMAX+ALPHA*LAMBDA*DELFTS
24            FORMAT(T3,'*',10X,'FCNMAX+ALPHA*LAMBDA*DELFTS:',
     $        ' ................',1PD12.3,T74,'*')
              IF(DEUFLH.OR.ACPTCR.EQ.12) THEN
		 IF(ITNUM .GT.0) THEN
		    IF(.NOT.SCLXCH) THEN
		       WRITE(NUNIT,1)
		       WRITE(NUNIT,25)
25                     FORMAT(T3,'*',10X,'DEUFLHARD SBAR VECTOR',
     $                 T74,'*')
		       WRITE(NUNIT,1)
		       DO 1500 I=1,N
			  WRITE(NUNIT,26) I,SBAR(I)
26                        FORMAT(T3,'*',10X,'SBAR(',I3,') = ',
     $                    1PD12.3,T74,'*')
1500                   CONTINUE
		    ELSE
		       WRITE(NUNIT,1)
		       WRITE(NUNIT,27)
27                     FORMAT(T3,'*',10X,'DEUFLHARD SBAR VECTOR',
     $                 14X,'IN SCALED X UNITS',T74,'*')
		       WRITE(NUNIT,1)
		       DO 1600 I=1,N
			  WRITE(NUNIT,28) I,SBAR(I),I,SCALEX(I)*SBAR(I)
28                        FORMAT(T3,'*',10X,'SBAR(',I3,') = ',1PD12.3,
     $                    8X,'SBAR(',I3,') = ',1PD12.3,T74,'*')
1600                   CONTINUE
		    END IF
		 END IF
              END IF
              IF(ACPTCR.EQ.12) THEN
		 WRITE(NUNIT,1)
		 IF(.NOT.SCLXCH) THEN
		    WRITE(NUNIT,29) SBRNRM
29                  FORMAT(T3,'*',10X,'VALUE OF SBRNRM',
     $              ' AT XPLUS: ..................'1PD12.3,T74,'*')
		 ELSE
		    WRITE(NUNIT,30) SBRNRM
30                  FORMAT(T3,'*',10X,'VALUE OF SCALED SBRNRM',
     $              ' AT XPLUS: ...........'1PD12.3,T74,'*')
		 END IF
		 WRITE(NUNIT,31) NEWMAX
31               FORMAT(T3,'*',10X,'NEWMAX:',' ..............',
     $           '......................',1PD12.3,T74,'*')
              END IF
           END IF
C
C          CHECK FOR ACCEPTABLE STEP.
C
           IF(FCNNEW.LT.FCNMAX+ALPHA*LAMBDA*DELFTS) THEN
              ACPCOD=1
C
C             NOTE: STEP WILL BE ACCEPTED REGARDLESS OF NEXT TEST.
C                   THIS SECTION IS FOR BOOKKEEPING ONLY.
C
              IF(ACPTCR.EQ.12) THEN
                 IF(SBRNRM.LT.NEWMAX) THEN
                    ACPCOD=12
                 END IF
              END IF
C
              RETURN
           END IF
           IF(ACPTCR.EQ.12.AND.SBRNRM.LT.NEWMAX) THEN
              ACPCOD=2
              RETURN
           END IF
C
C          FAILURE OF STEP ACCEPTANCE TEST.
C
           IF(CONVIO) THEN
              LAMBDA=CONFAC*RATIOM*LAMBDA
C
C             LAMBDA IS ALREADY SET - SKIP NORMAL PROCEDURE.
C
              GO TO 101
           END IF
           IF(LAMBDA.EQ.ZERO) THEN
              IF(OUTPUT.GT.0) THEN
                 WRITE(NUNIT,1)
		 WRITE(NUNIT,32)
32               FORMAT(T3,'*',7X,'LAMBDA IS 0.0:  NO PROGRESS',
     $           ' POSSIBLE - CHECK BOUNDS OR START',T74,'*')
                 ABORT=.TRUE.
                 RETURN
              END IF
           END IF
           IF(GEOMS) THEN
C
C             GEOMETRIC LINE SEARCH
C
              LAMBDA=SIGMA*LAMBDA
C
           ELSE
C
              IF(FRSTST) THEN
                 FRSTST=.FALSE.
C
C                FIND MINIMUM OF QUADRATIC AT FIRST STEP.
C
                 LAMTMP=-(LAMBDA*LAMBDA)*DELFTS/
     $                  (TWO*(FCNNEW-FCNOLD-LAMBDA*DELFTS))
                 IF(OUTPUT.GT.4) THEN
                    WRITE(NUNIT,1)
		    WRITE(NUNIT,33) LAMTMP
33                  FORMAT(T3,'*',13X,'TEMPORARY LAMBDA FROM',
     $              ' QUADRATIC MODEL:  ',1PD11.3,T74,'*')
                 END IF
C
              ELSE
C
C                FIND MINIMUM OF CUBIC AT SUBSEQUENT STEPS.
C
                 FACTOR=ONE/(LAMBDA-LAMPRE)
                 IF(LAMBDA*LAMBDA.EQ.ZERO) THEN
                    LAMBDA=SIGMA*LAMBDA
C
C                   NOTE: IF THIS LAMBDA**2 WAS ZERO ANY SUBSEQUENT
C                         LAMBDA**2 WILL ALSO BE ZERO.
C
                    GO TO 101
                 END IF
                 ACUBIC=FACTOR*((ONE/LAMBDA*LAMBDA)*(FCNNEW-FCNOLD-
     $           LAMBDA*DELFTS)-((ONE/LAMPRE*LAMBDA)*(FPLPRE-
     $           FCNOLD-LAMPRE*DELFTS)))
                 BCUBIC=FACTOR*((-LAMPRE/LAMBDA*LAMBDA)*(FCNNEW-FCNOLD-
     $           LAMBDA*DELFTS)+((LAMBDA/LAMPRE*LAMBDA)*(FPLPRE-
     $           FCNOLD-LAMPRE*DELFTS)))
                 IF(TWO*LOG10(ABS(BCUBIC)+EPS).GT.DBLE(MAXEXP))
     $              THEN
                    LAMTMP=SIGMA*LAMBDA
                    IF(OUTPUT.GT.4) THEN
                       WRITE(NUNIT,1)
		       WRITE(NUNIT,34)
34                     FORMAT(T3,'*',13X,'POTENTIAL OVERFLOW IN'
     $                 ' CALCULATING TRIAL LAMBDA FROM',T74,'*')
		       WRITE(NUNIT,35)
35                     FORMAT(T3,'*',13X,'CUBIC MODEL - LAMBDA',
     $                 ' SET TO SIGMA*LAMBDA',T74,'*')
                    END IF
                 ELSE
                    DISC=BCUBIC*BCUBIC-THREE*ACUBIC*DELFTS
                    IF(ABS(ACUBIC).LE.EPSMCH) THEN
                       LAMTMP=-DELFTS/(TWO*BCUBIC)
                    ELSE
                       IF(DISC.LT.ZERO) THEN
                          LAMTMP=SIGMA*LAMBDA
                       ELSE
                          LAMTMP=(-BCUBIC+SQRT(DISC))/(THREE*ACUBIC)
                       END IF
                    END IF
                    IF(OUTPUT.GT.4) THEN
                       WRITE(NUNIT,1)
		       WRITE(NUNIT,36) LAMTMP
36                     FORMAT(T3,'*',13X,'TEMPORARY LAMBDA FROM',
     $                 ' CUBIC MODEL: .....',1PD11.3,T74,'*')
                    END IF
                 END IF
                 IF(LAMTMP.GT.SIGMA*LAMBDA) THEN
                    LAMTMP=SIGMA*LAMBDA
                    IF(OUTPUT.GT.4) THEN
                       WRITE(NUNIT,1)
		       WRITE(NUNIT,37)
37                     FORMAT(T3,'*',13X,'LAMTMP TOO LARGE - REDUCED',
     $                 ' TO SIGMA*LAMBDA',T74,'*')
                    END IF
                 END IF
              END IF
              LAMPRE=LAMBDA
              FPLPRE=FCNNEW
              IF(LAMTMP.LT.POINT1*LAMBDA) THEN
                 IF(OUTPUT.GT.4) THEN
                    WRITE(NUNIT,1)
		    WRITE(NUNIT,38)
38                  FORMAT(T3,'*',13X,'LAMTMP TOO SMALL - INCREASED',
     $               ' TO 0.1*LAMBDA',T74,'*')
                 END IF
                 LAMBDA=POINT1*LAMBDA
              ELSE
                 IF(OUTPUT.GT.4.AND.LAMTMP.NE.SIGMA*LAMBDA) THEN
                    WRITE(NUNIT,1)
		    WRITE(NUNIT,39)
39                  FORMAT(T3,'*',13X,'LAMTMP WITHIN LIMITS - ',
     $              'LAMBDA SET TO LAMTMP',T74,'*')
                 END IF
                 LAMBDA=LAMTMP
              END IF
           END IF
101        CONTINUE
100     CONTINUE
C
C       FAILURE IN LINE SEARCH
C
        ACPCOD=0
C
C       IF A QUASI-NEWTON STEP HAS FAILED IN THE LINE SEARCH THEN
C       SET QNFAIL TO TRUE ANS RETURN TO SUBROUTINE NNES.  THIS WILL
C       CAUSE THE JACOBIAN TO BE RE-EVALUATED EXPLICITLY AND A LINE
C       SEARCH IN THE NEW DIRECTION CONDUCTED.
C
        IF(.NOT.RESTRT) THEN
           QNFAIL=.TRUE.
           RETURN
        END IF
C
C       FALL THROUGH MAIN LOOP - WARNING GIVEN.
C
        IF(OUTPUT.GT.2.AND.(.NOT.WRNSUP)) THEN
           WRITE(NUNIT,1)
	   WRITE(NUNIT,40) MAXLIN
40         FORMAT(T3,'*',7X,'WARNING: ',I3,' CYCLES COMPLETED',
     $     ' IN LINE SEARCH WITHOUT SUCCESS',T74,'*')
        END IF
        IF(STOPCR.EQ.2.OR.STOPCR.EQ.3) THEN
           STOPCR=12
           WRITE(NUNIT,1)
	   WRITE(NUNIT,41)
41         FORMAT(T3,'*',7X,'STOPPING CRITERION RESET FROM ',
     $     '2 TO 12 TO AVOID HANGING',T74,'*')
        END IF
        RETURN
C
C       LAST CARD OF SUBROUTINE DEUFLS.
C
        END
