        SUBROUTINE JACOBI(CHECKJ,JACERR,OVERFL,JACTYP,N     ,
     $                    NUNIT ,OUTPUT,EPSMCH,FDTOLJ,BOUNDL,
     $                    BOUNDU,FVECC ,FVECJ1,FVECJ2,JAC   ,
     $                    JACFDM,SCALEX,WV3   ,XC    ,FVECEV,
     $                    JACEV )
C
C       APR. 13, 1991
C
C       THIS SUBROUTINE EVALUATES THE JACOBIAN.  IF CHECKJ IS TRUE
C       THEN THE ANALYTICAL JACOBIAN IS CHECKED NUMERICALLY.
C
C       JACEV IS A USER-SUPPLIED ANALYTICAL JACOBIAN USED ONLY IF
C       JACTYP=0.  THE JACOBIAN NAME MAY BE CHANGED BY USING THE
C       EXTERNAL STATEMENT IN THE MAIN DRIVER.
C
C       JACFD ESTIMATES THE JACOBIAN USING FINITE DIFFERENCES:
C       FORWARD IF JACTYP=1 OR BACKWARD IF JACTYP=2.
C
C       JACCD ESTIMATES THE JACOBIAN USING CENTRAL DIFFERENCES.
C
C       IF THE ANALYTICAL JACOBIAN IS CHECKED THE FINITE DIFFERENCE
C       JACOBIAN IS STORED IN "JACFDM" AND THEN COMPARED.
C
C       FRSTER  INDICATES FIRST ERROR - USED ONLY TO SET BORDERS
C               FOR OUTPUT
C       JACERR  FLAG TO INDICATE TO THE CALLING PROGRAM AN ERROR
C               IN THE ANALYTICAL JACOBIAN
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DOUBLE PRECISION JAC(N,N) ,JACFDM(N,N)
        INTEGER          OUTPUT
	DIMENSION        BOUNDL(N) ,BOUNDU(N) ,FVECC(N) ,FVECJ1(N) ,
     $                   FVECJ2(N) ,SCALEX(N) ,WV3(N)   ,XC(N)
        LOGICAL          CHECKJ    ,FRSTER    ,JACERR   ,OVERFL
        EXTERNAL FVECEV,JACEV
        DATA ONE /1.0D0/
C
        FRSTER=.TRUE.
        JACERR=.FALSE.
        OVERFL=.FALSE.
C
        IF(JACTYP.EQ.0) THEN
           CALL JACEV(OVERFL,N,JAC,XC)
        ELSEIF(JACTYP.EQ.1.OR.JACTYP.EQ.2) THEN
           CALL JACFD(JACTYP,N     ,NUNIT ,OUTPUT,EPSMCH,BOUNDL,BOUNDU,
     $                FVECC ,FVECJ1,JAC   ,SCALEX,WV3   ,XC    ,FVECEV)
        ELSE
           CALL JACCD(N     ,NUNIT ,OUTPUT,EPSMCH,FVECJ1,
     $                FVECJ2,JAC   ,SCALEX,XC    ,FVECEV)
        END IF
C
        IF(JACTYP.EQ.0.AND.CHECKJ) THEN
C
C          NOTE: JACTYP=0 SENT TO JACFD PRODUCES A FORWARD
C                DIFFERENCE ESTIMATE OF THE JACOBIAN.
C
           CALL JACFD(JACTYP,N     ,NUNIT ,OUTPUT,EPSMCH,BOUNDL,
     $                BOUNDU,FVECC ,FVECJ1,JACFDM,SCALEX,WV3   ,
     $                XC    ,FVECEV)
           DO 100 J=1,N
              DO 200 I=1,N
                 IF(ABS((JACFDM(I,J)-JAC(I,J))/MAX(ABS(JAC(I,J)),
     $              ABS(JACFDM(I,J)),ONE)).GT.FDTOLJ) THEN
                    JACERR=.TRUE.
                    IF(OUTPUT.GE.0) THEN
                       IF(FRSTER) THEN
                          FRSTER=.FALSE.
                          WRITE(NUNIT,1)
1                         FORMAT(T3,72('*'))
                       END IF
                       WRITE(NUNIT,2)
2                      FORMAT(T3,'*',T74,'*')
                       WRITE(NUNIT,3) I,J
3                      FORMAT(T3,'*',4X,'CHECK JACOBIAN TERM (',I3,
     $                 ',',I3,')',T74,'*')
                       WRITE(NUNIT,2)
                       WRITE(NUNIT,4) JAC(I,J)
4                      FORMAT(T3,'*',4X,'ANALYTICAL DERIVATIVE IS ',
     $                 1PD12.3,T74,'*')
                       WRITE(NUNIT,5) JACFDM(I,J)
5                      FORMAT(T3,'*',4X,' NUMERICAL DERIVATIVE IS ',
     $                 1PD12.3,T74,'*')
                       WRITE(NUNIT,2)
                       WRITE(NUNIT,1)
                    END IF
                 END IF
200           CONTINUE
100        CONTINUE
        END IF
        RETURN
C
C       LAST CARD OF SUBROUTINE JACOBI.
C
        END
