        SUBROUTINE TWONRM(OVERFL,MAXEXP,N,EPSMCH,EUCNRM,V)
C
C       FEB. 23 ,1992
C
C       THIS SUBROUTINE EVALUATES THE EUCLIDEAN NORM OF A VECTOR, V.
C       IT FOLLOWS THE ALGORITHM OF J.L. BLUE IN ACM TOMS V4 15 (1978)
C       BUT USES SLIGHTLY DIFFERENT CUTS. THE CONSTANTS IN COMMON BLOCK
C       NNES_5 ARE CALCULATED IN THE SUBROUTINE MACHAR OR ARE PROVIDED
C       BY THE USER IN THE DRIVER.
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DIMENSION        V(N)
        LOGICAL          OVERFL
        COMMON/NNES_5/SMALLB,BIGB,SMALLS,BIGS,BIGR
        DATA ZERO,ONE,TWO,TEN /0.0D0,1.0D0,2.0D0,10.0D0/
C
        OVERFL=.FALSE.
        SQRTEP=SQRT(EPSMCH)
C
        ASMALL=ZERO
        AMED=ZERO
        ABIG=ZERO
        DO 100 I=1,N
C
C          ACCUMULATE SUMS OF SQUARES IN ONE OF THREE ACCULULATORS,
C          ABIG, AMED AND ASMALL, DEPENDING ON THEIR SIZES.
C
           ABSVI=ABS(V(I))
C
C          THIS COMPARISON RESTRICTS THE MAXIMUM VALUE OF AMED TO BE
C          B/N => CANNOT SUM SO THAT AMED OVERFLOWS.
C
           IF(ABSVI.GT.BIGB/DBLE(N)) THEN
C
C             THIS DIVISOR OF 10N RESTRICTS ABIG FROM (PATHALOGICALLY)
C             OVERFLOWING FROM SUMMATION.
C
	      ABIG=ABIG+((V(I)/(TEN*DBLE(N)*BIGS)))**2
C
           ELSEIF(ABSVI.LT.SMALLS) THEN
C
	      ASMALL=ASMALL+((V(I)/SMALLS))**2
C
           ELSE
C
	      AMED=AMED+V(I)*V(I)
C
           END IF
C
100     CONTINUE
        IF(ABIG.GT.ZERO) THEN
C
C          IF OVERFLOW WOULD OCCUR ASSIGN BIGR AS NORM AND SIGNAL TO
C          CALLING SUBROUTINE VIA OVERFL.
C
           IF(LOG10(ABIG)/TWO+LOG10(BIGS)+1+LOG10(DBLE(N))
     $        .GT.MAXEXP) THEN
              EUCNRM=BIGR
              OVERFL=.TRUE.
              RETURN
           END IF
C
C          IF AMED IS POSITIVE IT COULD CONTRIBUTE TO THE NORM -
C          DETERMINATION IS DELAYED UNTIL LATER TO SAVE REPEATING
C          CODE.
C
           IF(AMED.GT.ZERO) THEN
              YMIN=MIN(SQRT(AMED),TEN*DBLE(N)*BIGS*SQRT(ABIG))
              YMAX=MAX(SQRT(AMED),TEN*DBLE(N)*BIGS*SQRT(ABIG))
           ELSE
C
C             AMED DOESN'T CONTRIBUTE AND ASMALL WON'T MATTER IF
C             ABIG IS NONZERO - FIND NORM USING ABIG AND RETURN.
C
              EUCNRM=TEN*DBLE(N)*BIGS*SQRT(ABIG)
              RETURN
           END IF
        ELSEIF(ASMALL.GT.ZERO) THEN
           IF(AMED.GT.ZERO) THEN
              YMIN=MIN(SQRT(AMED),SMALLS*SQRT(ASMALL))
              YMAX=MAX(SQRT(AMED),SMALLS*SQRT(ASMALL))
           ELSE
C
C             ABIG AND AMED ARE ZERO SO USE ASMALL ONLY.
C
              EUCNRM=SMALLS*SQRT(ASMALL)
              RETURN
           END IF
        ELSE
           EUCNRM=SQRT(AMED)
           RETURN
        END IF
        IF(YMIN.LT.SQRTEP*YMAX) THEN
C
C          SMALLER PORTION DOES NOT CONTRIBUTE TO NORM.
C
           EUCNRM=YMAX
        ELSE
C
C          SMALLER PORTION CONTRIBUTES TO NORM.
C
           EUCNRM=YMAX*SQRT((ONE+YMIN/YMAX)*(ONE+YMIN/YMAX))
        END IF
        RETURN
C
C       LAST CARD OF SUBROUTINE TWONRM.
C
        END
