// This is the nonlinear solver interface class (nnes)
// by Carlos E. Christoffersen

#ifndef NLSInterface_h
#define NLSInterface_h 1

#include "OFunction.h"

class NLSInterface
{
	public:
  NLSInterface(OFunction* of, int& sysdim, int diff=1);
  
  ~NLSInterface();
  
  int solve(double* X, double& res);
	
  friend void jacFunc(int *overfl, int *n1, double* J, double* X);
	
  friend void nlfcn(int *overfl, int *n1, double* F, double* X);
	
	private:
  // Static pointer to the instance 
  static NLSInterface* nlsi;
	
  // pointer to the objective function class
  OFunction* of;
	
  // variables used by nnes
  int i;
  int n, mgll, nunit;
  double lam0, mstpf;
  int acptcr,  output, qnupdm, stopcr, supprs, trmcod, trupdm;
  
  // vectors and arrays 
  double *jac,*a ,*boundl ,*boundu ,*delf ,*fsave ,*ftrack ,*fvec ,*fvecc ,*h,
	*hhpi ,*plee,*rdiag ,*s ,*sbar ,*scalef,*scalex ,*sn ,*ssdhat ,*strack,
	*vhat ,*wv1 ,*wv2 ,*wv3 ,*wv4 ,*xplus ,*xsave;
	
  // logical variables  
  int absnew ,bypass ,cauchy ,deuflh , geoms, linesr ,matsup ,
	newton , overch , overfl;
	
  char help[7];
	
  // implicitly-defined variables.
  int itsclf, itsclx, jactyp, jupdm, maxexp, maxit, maxns, maxqns,
	minqns, narmij, niejev, njacch;
	
  double alpha, confac, delta, delfac, epsmch, etafac, fdtolj, ftol,
	nsttol, omega, ratiof, sigma, stptol, njetot, fcnnew;
};

#endif

