// Class definition for the new general state variable transient analysis
//
// Author:
//          Sonali R. Luniya
//

#ifndef SVTran4_h
#define SVTran4_h 1

#include "Analysis.h"
#include "OFunction.h"
#include "CircVector.h"
#include "../freeda_mtl.h"
extern "C" {
#undef _S
#undef _C
#include "dsp_defs.h"
#include "util.h"
	   }

// Main class definition follows
class SVTran4 : public Analysis, public OFunction
{
public:

  SVTran4();

  ~SVTran4();

  // (from Analysis)
  // The main analysis routine.
  virtual void run(Circuit* cir);
  virtual void dummy_run(Circuit* cir);
  //Return the name of the element
  static const char* getNetlistName()
  {
     return ainfo.name;
  } 
   
  // (from OFunction)
  // Evaluate error function vector F given X
  virtual void func_ev(double* X, double* F);
  // Evaluate Jacobian
  virtual void jacobian(double* X, DoubleDenseColMatrix& J);

  // Write output vectors
  void doOutput();
 
private:

  void superLUFactor(DoubleSparseColMatrix *& M, DenseDoubleVector& ssv);
  void freeSuperLU();
  void f1(DenseDoubleVector& s1, DenseDoubleVector& v1l);
  void updateVInl(double* x_p);
  void multiply(DoubleDenseColMatrix& a , ExtDoubleVector& b, DenseDoubleVector& c);
  void updateU(DenseDoubleVector& s1);
  void buildMsv();
  int PredictNewStep(DenseDoubleVector& ssv, double *timestep);
  Circuit* cir;
  int n_states, n_sec_states;
  IntDenseColMatrix T;
  ElementVector elem_vec;
  DenseIntVector cns, nss;    // Vectors for number of primary and secondary SV.
  TimeDomainSV *tdsv;
  int ls_size;
  int n_tsteps;
  int list_size;
  DenseDoubleVector ssv, vl1, itmp2, tmp_v, rhs_v, c_v, r_v;
  CircVector *cU;
  //CircVector *cX; //Have this only if the *cX is not a global array.
  CircVector *cY;
  CircVector *cVnl;
  CircVector *cInl;
  CircVector *cTime;
  CircVector *cTimestep;
  
  DoubleDenseRowMatrix Msv;

  // SuperLU variables
  SuperMatrix A, L, U, B, X;
  int *perm_r;
  int *perm_c;
  int *etree;
  double *r, *c;
  double *rhs_p;
  void *work;
  int lwork;
  bool superLU;
  double* elem_val;
  double* rhs_Jv_X;
  double* Integ_predX;
  double* Integ_predU;
  int* row_index;
  int* col_pointer;

  // ------------------- Parameter-related variables
  // Analysis information
  static ItemInfo ainfo;
  // Number of parameters of this element
  static const unsigned n_par;
  // Analysis parameters
  double h, tf, nst, gcomp,cm,abstol,reltol,tol,tmin;
  int deriv, int_method, permc_spec, out_steps,max_iter;
  bool use_msv, savenode;

  DenseDoubleVector rcomp;
  CircVector *cX;
  int nt ,total_iteration, scount , col_count, mnam_scount,cmflag,cktorder;
  double prev_timestep, new_timestep, ptime, net_timestep, sparsity;

  // Parameter information
  static ParmInfo pinfo[];
};


#endif
