// This class stores the state variables, currents and voltages resulting
// from the wavelet simulation.
//
// by Carlos E. Christoffersen

#ifndef WTDBase_h
#define WTDBase_h 1

#include "WTLinear.h"

class WTDBase
{
	public:

  WTDBase(ElementVector& elem_vec, const int& nstates,
	const int& nwindows);

  ~WTDBase();

  // Set current time window
  inline void setWindow(const int& n,
	const int& nsamples, const int lastsample)
  {
    nw = n;
    this->nsamples = nsamples;
    this->lastsample = lastsample;
  }

  // Get initial conditions for x
  void getXIC(DenseDoubleVector& x_ic);

  // Get initial conditions for u
  void getUIC(DenseDoubleVector& u_ic);

  // Set results for the current window.
  void saveWindow(WTLinear *wtl, ExtDoubleVector& x,
	DenseDoubleVector& vp, DenseDoubleVector ip);

  // Write results to output tables.
  void writeOutput();

	private:

  ElementVector elem_vec;
  int nstates;
  int nwindows;
  int nw;
  int nsamples;
  int lastsample;
  DenseDoubleVector *timev, *xw, *vpw, *ipw, *uw;
};

#endif

