// This class handles the linear system in the Wavelet transient.
//
// Author:
//          Carlos E. Christoffersen
//

#ifndef WTNonLinear_h
#define WTNonLinear_h 1

#include "WaveletBase.h"
#include "OFunction.h"
#include "../network/Circuit.h"
#include "WTLinear.h"
#include "WaveletDomainSV.h"
#include "WTDBase.h"
#include "NLSInterface.h"

// Main class definition follows

class WTNonLinear : public OFunction
{
	public:

  WTNonLinear(WaveletBase* wb, ElementVector& elem_vec,
	int& n_states, int& max_n_states,
	WTLinear* wtl, WTDBase* wtr);

  ~WTNonLinear();

  // Solve the nonlinear system
  int solve(const int& nsteps);

  // Evaluate error function vector F given X
  virtual void func_ev(double* X, double* F);
  // Evaluate Jacobian
  virtual void jacobian(double* X, DoubleDenseExtColMatrix& J);

	private:

  void getVI(ExtDoubleVector& xv);

  WaveletBase* wb;
  ElementVector elem_vec;
  int n_states, max_n_states;
  WTLinear* wtl;
  WTDBase* wtr;
  int npoints, sysdim;
  DenseDoubleVector vNL, iNL, vL;
  DoubleDenseColMatrix JacVNL, JacINL;
  WaveletDomainSV* wdsv;
  double *x;
  DenseDoubleVector xic;
  NLSInterface* nlsi;
};

#endif

