#include "WaveletDomainSV.h"

WaveletDomainSV::WaveletDomainSV(WaveletBase* wb, const int& max_n_states,
DenseDoubleVector& v, DenseDoubleVector& i,
DenseDoubleVector& xic) :
wb(wb), nosamples(wb->getSize()-1), max_n_states(max_n_states),
base(0), cns(1), vv(v), iv(i), xic(xic),
JacVp(max_n_states * nosamples, max_n_states * nosamples),
JacIp(max_n_states * nosamples, max_n_states * nosamples)
{
  wb->getWJr(Wj);
  wb->getWpJr(Wjp);

  xt = new DenseDoubleVector[max_n_states];
	for (int i = 0; i < max_n_states; i++)
      xt[i] = DenseDoubleVector(nosamples);

  xdt = new DenseDoubleVector[max_n_states];
	for (int i = 0; i < max_n_states; i++)
      xdt[i] = DenseDoubleVector(nosamples);

  // Create workspace for jacobian calculation
  mv_jvp = new DenseDoubleVector[max_n_states * max_n_states];
	for (int i = 0; i < max_n_states*max_n_states; i++)
    mv_jvp[i] = DenseDoubleVector(nosamples);

  mv_jip = new DenseDoubleVector[max_n_states * max_n_states];
	for (int i = 0; i < max_n_states*max_n_states; i++)
    mv_jip[i] = DenseDoubleVector(nosamples);

  mv_jdvp = new DenseDoubleVector[max_n_states * max_n_states];
	for (int i = 0; i < max_n_states*max_n_states; i++)
    mv_jdvp[i] = DenseDoubleVector(nosamples);

  mv_jdip = new DenseDoubleVector[max_n_states * max_n_states];
	for (int i = 0; i < max_n_states*max_n_states; i++)
    mv_jdip[i] = DenseDoubleVector(nosamples);
}

WaveletDomainSV::~WaveletDomainSV()
{
  delete [] xdt;
  delete [] xt;

  delete [] mv_jdip;
  delete [] mv_jdvp;
  delete [] mv_jip;
  delete [] mv_jvp;
}

void WaveletDomainSV::setBase(const int& base, const int& cns)
{
  this->base = base;
  this->cns = cns;
  // Transform X to time domain
  // Do this here only by now...
  for (int i=0; i<cns; i++) {
    ExtDoubleVector aux(&(xv[0]) + (base+i)*nosamples, nosamples);
    wb->toTime(xic[base+i], aux, xt[i]);
    wb->toTimeDeriv(xic[base+i], aux, xdt[i]);
  }
}


void WaveletDomainSV::addJac(const int& js,
DoubleDenseColMatrix& Jac, DenseDoubleVector*& mv_Jac,
DoubleSparseColMatrix& W)
{
  for (int i=0; i < cns; i++) {
    // Scale each row of W by the elements in mv_Jac[max_n_states*i+js]
    DoubleSparseColMatrix::iterator i_sp, i_spend;
    DoubleSparseColMatrix::OneD::iterator j_sp, j_spend;
    i_sp = W.begin();
    i_spend = W.end();
    for (; i_sp != i_spend; ++i_sp) {
      j_sp = (*i_sp).begin();
      j_spend = (*i_sp).end();
      for (; j_sp != j_spend; ++j_sp) {
				// j_sp.row(), j_sp.column(), (*j_sp)
				Jac(nosamples * i + j_sp.row(),
				nosamples * js + j_sp.column()) +=
				(*j_sp) * mv_Jac[max_n_states * i + js][j_sp.row()];
      }
    }
  }
}

void WaveletDomainSV::cleanJac()
{
  for (int i(0); i < nosamples * cns; i++)
	for (int j(0); j < nosamples * cns; j++) {
		JacVp(i,j) = zero;
		JacIp(i,j) = zero;
	}
}
