// This is the standard interface of wavelet domain state variable
// analysis with the elements.
// This class is a quick hack only!!!
// It must be rewritten whenever is possible.
// by Carlos E. Christoffersen

#ifndef WaveletDomainSV_h
#define WaveletDomainSV_h 1

#include "WaveletBase.h"
#include "../freeda_mtl.h"

class WaveletDomainSV
{
	public:

  WaveletDomainSV(WaveletBase *wb, const int& max_n_states,
	DenseDoubleVector& v, DenseDoubleVector& i,
	DenseDoubleVector& xic);

  ~WaveletDomainSV();

  inline const int noSamples() const
  {
    return nosamples;
  }

  // Get x_i (time domain vector)
  inline DenseDoubleVector& getx(const int& i)
  {
    return xt[i];
  }
  // Get time domain derivative
  inline DenseDoubleVector& getdx_dt(const int& i)
  {
    return xdt[i];
  }
  // Port voltage (nstate, ntimestep)
  inline double& vp(const int&i, const int& j)
  {
    return vv[(base + i) * nosamples + j];
  }
  // Port current (nstate, ntimestep)
  inline double& ip(const int&i, const int& j)
  {
    return iv[(base + i) * nosamples + j];
  }

  // Get time-domain partial Jacobian diagonals
  inline DenseDoubleVector& Jvp(const int& i,const int& j)
	{
		return mv_jvp[max_n_states*i+j];
	}
  inline DenseDoubleVector& Jip(const int& i,const int& j)
	{
		return mv_jip[max_n_states*i+j];
	}
  inline DenseDoubleVector& Jvpdx_dt(const int& i,const int& j)
	{
		return mv_jdvp[max_n_states*i+j];
	}
  inline DenseDoubleVector& Jipdx_dt(const int& i,const int& j)
	{
		return mv_jdip[max_n_states*i+j];
	}

  // The following methods require the Jacobians already initialized
  // js: differentiation variable index.
  inline void addJ(const int& js)
	{
		addJac(js, JacVp, mv_jvp, Wj);
		addJac(js, JacIp, mv_jip, Wj);
	}
  inline void addJdx_dt(const int& js)
	{
		addJac(js, JacVp, mv_jdvp, Wjp);
		addJac(js, JacIp, mv_jdip, Wjp);
	}

  // Clean the Jacobian block associated with the current element.
  void cleanJac();

  // Methods used by the analysis
  inline void setX(ExtDoubleVector& x)
  {
    this->xv = x;
  }

  void setBase(const int& base, const int& cns);

  // returns jacobian block matrix
  inline DoubleDenseColMatrix& getJacVp()
  {
    return JacVp;
  }
  // returns jacobian block matrix
  inline DoubleDenseColMatrix& getJacIp()
  {
    return JacIp;
  }

	private:

  void addJac(const int& js,
	DoubleDenseColMatrix& Jac, DenseDoubleVector*& mv_Jac,
	DoubleSparseColMatrix& W);

  WaveletBase* wb;
  DoubleSparseColMatrix Wj, Wjp;
  int nosamples, max_n_states;
  int base, cns;
  DenseDoubleVector vv, iv, xic;
  DoubleDenseColMatrix JacVp, JacIp;
  DenseDoubleVector *xt, *xdt;
  bool ready;
  ExtDoubleVector xv;
  DenseDoubleVector *mv_jvp, *mv_jip, *mv_jdvp, *mv_jdip;
};

#endif

