/*********************************************************************/
/*                           Spring 2003                             */
/*                              ECE718                               */
/*                                                                   */
/*               Digital to Analog Circuit Converter                 */
/*                                                                   */
/*                      by Mazen M Kharbutli                         */
/*                    mazen_kharbutli@yahoo.com                      */
/*                                                                   */
/* This program creates a fReeda compatable netlist that includes    */
/*    diodes, transistors, and resistors from a netlist with digital */
/*    gates.                                                         */
/*                                                                   */
/*********************************************************************/

#include<iostream.h>
#include<fstream.h>
#include<io.h>
#include<stdlib.h>
#include<string.h>

#define MAX_NUM_OF_NODES 1024

// The converter class
class Digital_to_Analog_Converter
{
   private:
      int total_gates;    // Total number of gates in circuit
      int current_node;   // Current node number
      int current_resistor, current_diode, current_transistor;
      ifstream fin;
      ofstream fout;

   public:
      Digital_to_Analog_Converter() {;}
      ~Digital_to_Analog_Converter() {;}

      void initialize(char input[], char output[]);
      void Diode(int n1, int n2);
      void Resistor(int n1, int n2, int val);
      void BJT(int n1, int n2, int n3);
      void NMOS(int n1, int n2, int n3);
      void PMOS(int n1, int n2, int n3);
      void TTL_INV(int in, int out, int H, int L);
      void TTL_NOR(int in1, int in2, int out, int H, int L);
      void TTL_NAND(int in1, int in2, int out, int H, int L);
      void CMOS_INV(int in, int out, int H, int L);
      void CMOS_NOR(int in1, int in2, int out, int H, int L);
      void CMOS_NAND(int in1, int in2, int out, int H, int L);
      void get_gates();
      void finish();
};

// This function opens the input and output files and initializes the converter
void Digital_to_Analog_Converter::initialize(char input[], char output[])
{
   char temp[80], tempc;

   current_node=MAX_NUM_OF_NODES;
   current_resistor=0;
   current_transistor=0;
   current_diode=0;
   total_gates=0;

   cout<<"\nWelcome to the Digital to Analog Converter Program."<<endl;

   fin.open(input);
   fout.open(output);

   fout<<"\n* This file was produced by the Digital to Analog Circuit Converter.";
   fout<<"\n* The converter program was written by Mazen M Kharbutli.\n";
   fout<<endl;

   fin>>ws;
   fin>>temp;
   while(strcmp(temp,".START") != 0)
   {
     fout<<temp;
     while(fin.peek()!='.')
     {
        fin.get(tempc);
        fout.put(tempc);
     }
     fin>>temp;
   }

}

// This function completes the conversion and closes the files
void Digital_to_Analog_Converter::finish()
{
   char temp, temp1[32];

   fin>>temp1;

   fin>>ws;
   while(fin.peek()!=EOF)
   {
      fin.get(temp);
      fout.put(temp);
   }

   fin.close();
   fout.close();

   cout<<"\nDone converting the file."<<endl;
   cout<<"\nNumber of Gates = "<<total_gates;
   cout<<"\nNumber of Resistors = "<<current_resistor;
   cout<<"\nNumber of Diodes = "<<current_diode;
   cout<<"\nNumber of Transistors = "<<current_transistor<<endl;
}

// This function writes out a diode between n1 (+) and n2 (-)
void Digital_to_Analog_Converter::Diode(int n1, int n2)
{
   fout<<"\nd:D"<<current_diode<<" "<<n1<<" "<<n2
       <<" vj=1.0 bv=75 tt=30e-9";
   current_diode++;
}

// This function writes out a resistor between n1 and n2 with resistance (val)
void Digital_to_Analog_Converter::Resistor(int n1, int n2, int val)
{
   fout<<"\nres:R"<<current_resistor<<" "<<n1<<" "<<n2<<" r="<<val;
   current_resistor++;
}

// This function writes out a BJT npn transistor n1:C n2:B n3:E
void Digital_to_Analog_Converter::BJT(int n1, int n2, int n3)
{
   fout<<"\nbjtnpn:T"<<current_transistor<<" "<<n1<<" "<<n2<<" "<<n3<<" "<<n3
       <<" tf=2e-9 tr=6e-9 vje=0.7 vjc=0.6 br=0.1 bf=100";
   current_transistor++;
}

// This function writes out a NMOS transistor n1:D n2:G n3:S
void Digital_to_Analog_Converter::NMOS(int n1, int n2, int n3)
{
   fout<<"\nnmos3:T"<<current_transistor<<" "<<n1<<" "<<n2<<" "<<n3<<" "<<n3
       <<" kp=0.002";
   current_transistor++;
}

// This function writes out a PMOS transistor n1:D n2:G n3:S
void Digital_to_Analog_Converter::PMOS(int n1, int n2, int n3)
{
   fout<<"\npmos3:T"<<current_transistor<<" "<<n1<<" "<<n2<<" "<<n3<<" "<<n3
       <<" kp=0.002";
   current_transistor++;
}

// This fuction converts a TTL inverter to an analog circuit
void Digital_to_Analog_Converter::TTL_INV(int in, int out, int H, int L)
{
   Diode(L,in);
   Diode(current_node+4,out);

   Resistor(H,current_node,4000);
   Resistor(H,current_node+2,1600);
   Resistor(H,current_node+5,130);
   Resistor(L,current_node+3,1000);

   BJT(current_node+1,current_node,in);
   BJT(current_node+2,current_node+1,current_node+3);
   BJT(current_node+5,current_node+2,current_node+4);
   BJT(out,current_node+3,L);

   current_node=current_node+6;
   total_gates++;
}

// This fuction converts a CMOS inverter to an analog circuit
void Digital_to_Analog_Converter::CMOS_INV(int in, int out, int H, int L)
{
   NMOS(out,in,L);
   PMOS(out,in,H);

   total_gates++;
}

// This fuction converts a TTL NOR gate to an analog circuit
void Digital_to_Analog_Converter::TTL_NOR(int in1, int in2, int out, int H, int L)
{
   Diode(current_node+2,current_node+3);
   Diode(current_node+4,current_node+5);

   Resistor(H,current_node,4700);
   Resistor(H,out,1000);
   Resistor(H,current_node+1,4700);

   BJT(current_node+2,current_node,in1);
   BJT(current_node+4,current_node+1,in2);
   BJT(out,current_node+3,L);
   BJT(out,current_node+5,L);

   current_node=current_node+6;
   total_gates++;
}

// This fuction converts a CMOS NOR gate to an analog circuit
void Digital_to_Analog_Converter::CMOS_NOR(int in1, int in2, int out, int H, int L)
{
   NMOS(out,in1,L);
   NMOS(out,in2,L);
   PMOS(out,in1,current_node);
   PMOS(current_node,in2,H);

   current_node++;
   total_gates++;
}

// This fuction converts a TTL NAND gate to an analog circuit
void Digital_to_Analog_Converter::TTL_NAND(int in1, int in2, int out, int H, int L)
{
   Diode(current_node+1,current_node+2);

   Resistor(H,current_node,4700);
   Resistor(H,out,4700);
   Resistor(L,current_node+2,1000);

   BJT(current_node+1,current_node,in1);
   BJT(current_node+1,current_node,in2);
   BJT(out,current_node+2,L);

   current_node=current_node+3;
   total_gates++;
}

// This fuction converts a CMOS NAND gate to an analog circuit
void Digital_to_Analog_Converter::CMOS_NAND(int in1, int in2, int out, int H, int L)
{
   NMOS(out,in1,current_node);
   NMOS(current_node,in2,L);
   PMOS(out,in1,H);
   PMOS(out,in2,H);

   current_node++;
   total_gates++;
}

void Digital_to_Analog_Converter::get_gates()
{
   int n1, n2, n3, n4, n5;
   char name[32];

   while(fin.peek()!='.')
   {
      fin>>name;
      if(strcmp(name,"ttlinv")==0)
      {
         fin>>n1>>n2>>n3>>n4;
         TTL_INV(n1,n2,n3,n4);
         fout<<endl;
      }

      if(strcmp(name,"cmosinv")==0)
      {
         fin>>n1>>n2>>n3>>n4;
         CMOS_INV(n1,n2,n3,n4);
         fout<<endl;
      }

      if(strcmp(name,"ttlnand")==0)
      {
         fin>>n1>>n2>>n3>>n4>>n5;
         TTL_NAND(n1,n2,n3,n4,n5);
         fout<<endl;
      }

      if(strcmp(name,"ttlnor")==0)
      {
         fin>>n1>>n2>>n3>>n4>>n5;
         TTL_NOR(n1,n2,n3,n4,n5);
         fout<<endl;
      }

      if(strcmp(name,"cmosnand")==0)
      {
         fin>>n1>>n2>>n3>>n4>>n5;
         CMOS_NAND(n1,n2,n3,n4,n5);
         fout<<endl;
      }

      if(strcmp(name,"cmosnor")==0)
      {
         fin>>n1>>n2>>n3>>n4>>n5;
         CMOS_NOR(n1,n2,n3,n4,n5);
         fout<<endl;
      }

      //while(fin.peek()!='\n') fin.get(temp);
      fin>>ws;
   }

   cout<<"\nDone reading gates."<<endl;
}

main(int argc, char *argv[])
{
   char input[16], output[16];

   strcpy(input,argv[1]);
   strcpy(output,argv[2]);

   Digital_to_Analog_Converter converter;
   converter.initialize(input,output);
   converter.get_gates();
   converter.finish();
}
