// MOSFET LEVEL 3 MODEL
//
//          Drain 2
//                  o
//                  |
//                  |
//              |---+
//              |
// Gate 1 o-----|------o 4 Bulk
//              |
//              |---+
//                  |
//                  |
//                  o
//         Source 3
//
//
//	  Author:
//		Nikhil Kriplani

#ifndef Mosp3_h
#define Mosp3_h 1

class Mosp3 : public AdolcElement2
{
	public:

  Mosp3(const string& iname);

  ~Mosp3() {}

  static const char* getNetlistName()
  {
    return einfo.name;
  }

  // Do some local initialization
  virtual void init() throw(string&);

	private:

  virtual void eval1(adoublev& x, adoublev& xt, adoublev& y1, adoublev& z1);
  virtual void eval2(adoublev& dy1, adoublev& z1, adoublev& vp, adoublev& ip);

  // Some constants
  double pi, epsilon0, epsilons, q, leff, weff, vt, cox, oxidecap;

  // Element information
  static ItemInfo einfo;

  // Number of parameters of this element
  static const unsigned n_par;

  // Parameter variables
  double GAMMA, KP, LD, NSUB;
  double PHI, TOX, U0, VT0, WD, W, L, T, TNOM, NFS;
  double ETA, THETA, VMAX, KAPA, XJ, DELTA;
  double NSS, TPG, KAPPA;

  // Parameter information
  static ParmInfo pinfo[];

};

#endif

