// EKV 2.6 MOSFET MODEL WITH ELECTRO-THERMAL
//
//                Drain 1
//                  o
//                  |
//                  |
//              |---+
//              |     		   o 5 thermal node
// Gate 2 o-----|------o 4 Bulk        
//              |     
//              |---+		   o 6 thermal reference node
//                  |
//                  |
//                  o
//               Source 3
//
//
//

#ifndef Mospekvt_h
#define Mospekvt_h 1

#define NMOS 1
#define PMOS -1
class Mospekvt:public AdolcElement2
{
public:

  Mospekvt(const string& iname);
  ~Mospekvt();

  static const char* getNetlistName()
  {
    return einfo.name;
  }

  // Do some local initialization
  virtual void init() throw(string&);
  virtual void getLocalRefIdx(UnsignedVector& local_ref_vec,TerminalVector& term_list);

private:

  virtual void eval1(adoublev& x, adoublev& xt, adoublev& y1, adoublev& z1);
  virtual void eval2(adoublev& dy1, adoublev& z1, adoublev& vp, adoublev& ip);

  // Element information
  static ItemInfo einfo;

  // Number of parameters of this element
  static const unsigned n_par;

  double epsilonsi, epsilonox, q, k, Tref, /*vtT,*/ vtTnom, vtTref, /*egT,*/ egTnom;
  double egTref, /*niT,*/ niTnom, eta, vtoT, kpT, ucritT, phiT, ibbT;

  // Parameter variables
  double type, l, w, np, ns, cox, xj, dw, dl, vto, gamma, phi, kp, eo;
  double ucrit, tox, nsub, vfb, uo, vmax, theta, lambda, weta, leta, qo;
  double lk, iba, ibb,ibn, tcv, bex, ucex, ibbt, avto, akp, agamma, kf; 
  double af, nqs, satlim, xqc, scale, tnom, tmp;

  //added heat flux
  //double hf;
  //static int i;

  // Parameter information
  static ParmInfo pinfo[];

    // pointer to the interpolation routine
  double * voltages;
  double * currents;
  int length;

  // function to calculate linear interpolated value
  adouble linearInterpolation(const double voltage);

  // return interpolated value
  adouble linearInterpolate(const adouble xval);
  int bSearch(double xval, int index_lo, int index_hi);
  int accelFind(double xval, int len);
};

#endif //_Mospekvt_H_
