// Modified for fREEDA 1.3 compliance FPH 1/6/08
// Modified for fREEDA 1.2 compliance FPH 9/5/06
// This is a fREEDA(tm) element FPH 2/6/03
#include "../../../../network/ElementManager.h"
#include "../../../../network/AdolcElement.h"
#include "../../../../analysis/FreqMNAM.h"
#include "../../../../analysis/TimeMNAM.h"
#include <math.h>
#include "Vccsd.h"

// Static members
const unsigned Vccsd::n_par = 3 ;

// Element information
ItemInfo Vccsd::einfo = {
  "vccsd",
  "Volt-ctrld curr src with delay (for TWT Xsubckt)",
  "Frank P. Hart",
  DEFAULT_ADDRESS"elements/Vccsd.h.html"
};

// Parameter information
ParmInfo Vccsd::pinfo[] = {
  {"n", "Input Voltage Exponent ()",TR_DOUBLE, false},
  {"g", "Transconductance (Amps/Volts^n)", TR_DOUBLE, false},
  {"td", "Input Voltage Time Delay (Seconds)",TR_DOUBLE, false}
};


Vccsd::Vccsd(const string& iname) : AdolcElement(&einfo, pinfo, n_par, iname)
{
  // Value required
  paramvalue[0] = &(n=1.0) ;
  paramvalue[1] = &(g=1.0) ;
  paramvalue[2] = &(td=0.0) ;
  
  // Set flags
  setFlags(NONLINEAR | MULTI_REF | TR_TIME_DOMAIN);
}

void Vccsd::getLocalRefIdx(UnsignedVector& local_ref_vec,
			    TerminalVector& term_list)
{
  // Make sure the vectors are empty
  term_list.erase(term_list.begin(), term_list.end());
  local_ref_vec.erase(local_ref_vec.begin(), local_ref_vec.end());

  // Insert vector elements
  term_list.push_back(getTerminal(0)); // Input port terminal 
  term_list.push_back(getTerminal(1)); // Input port local reference terminal
  local_ref_vec.push_back(1); // Local reference index

  term_list.push_back(getTerminal(2)); // Output port terminal
  term_list.push_back(getTerminal(3)); // Output port local reference terminal
  local_ref_vec.push_back(3); // Local reference index
}

void Vccsd::init() throw(string&)
{
  if(!isSet(&n) || !isSet(&g) || !isSet(&td)) {
    throw(" Not Enough Parameters: ") ;
/*    if !(isSet(&n)) throw("n, Voltage exponent, must be specified");
    if !(isSet(&g)) throw("g, Transconductance parameter, must be specified");
    if !(isSet(&td)) throw("td, Time Delay parameter, must be specified");
*/
  }
  else {

    // Set the number of terminals
    setNumTerms(4) ;

    // Set number of states
    setNumberOfStates(2);

    // create tape
    DenseIntVector var(2,0);
    var[1]=1;
    DenseIntVector novar;
    DenseIntVector tvar(1,0);
    DenseDoubleVector tdelay(1,td);
    createTape(var, novar, novar, tvar, tdelay) ;
  }
}

void Vccsd::eval(adoublev& x,
	       adoublev& vp, adoublev& ip)
{
  // x[0] = v_in(t)
  // x[1] = v_out(t)
  // x[2] = v_in(t-td)
  vp[0] = x[0] ;
  ip[0] = zero ;      // Should this be 0?  It's not really 0...	
  
  ip[1] = -g*pow(x[2],n) ; // i = g*v_in(t-td)^n --> This needs '-' sign
                      //    to fit the current convention!   
  vp[1] = x[1] ;      // ? HKanj - needed degree of freedom.
}
