// Modified for fREEDA 1.3 compliance FPH 1/7/08
// Modified for fREEDA 1.2 compliance FPH 9/5/06
// This is a fREEDA(tm) element FPH 5/17/04
#include "../../../../network/ElementManager.h"
#include "../../../../network/AdolcElement.h"
#include "../../../../analysis/FreqMNAM.h"
#include "../../../../analysis/TimeMNAM.h"
#include <math.h>
#include "Vctb_cann.h"

// Static members
const unsigned Vctb_cann::n_par = 3 ;

// Element information
ItemInfo Vctb_cann::einfo = {
  "vctb_cann",
  "Volt-to-curr transducer with Cann behavior",
  "Frank P. Hart",
  DEFAULT_ADDRESS"elements/Vctb_cann.h.html"
};

// Parameter information
ParmInfo Vctb_cann::pinfo[] = {
  {"gain", "Linear Gain (Amps/Volts)",TR_DOUBLE, false},
  {"limit", "Output Limit Current (Amps)",TR_DOUBLE, false},
  {"s", "Sharpness Parameter ()",TR_DOUBLE, false},
};


Vctb_cann::Vctb_cann(const string& iname) : AdolcElement(&einfo, pinfo, n_par, iname)
{
  // Value required
  paramvalue[0] = &(gain=1.0) ;
  paramvalue[1] = &(limit=1.0) ;
  paramvalue[2] = &(s=1.0) ;
  
  // Set flags
  setFlags(NONLINEAR | MULTI_REF | TR_TIME_DOMAIN);
}

void Vctb_cann::getLocalRefIdx(UnsignedVector& local_ref_vec,
			    TerminalVector& term_list)
{
  // Make sure the vectors are empty
  term_list.erase(term_list.begin(), term_list.end());
  local_ref_vec.erase(local_ref_vec.begin(), local_ref_vec.end());

  // Insert vector elements
  term_list.push_back(getTerminal(0)); // Input port terminal 
  term_list.push_back(getTerminal(1)); // Input port local reference terminal
  local_ref_vec.push_back(1); // Local reference index

  term_list.push_back(getTerminal(2)); // Output port terminal
  term_list.push_back(getTerminal(3)); // Output port local reference terminal
  local_ref_vec.push_back(3); // Local reference index
}

void Vctb_cann::init() throw(string&)
{
  if(isSet(&limit) && (limit<0.0)) {
    throw(" Limit must be positive! ") ;
  }
  else if(isSet(&s) && (s<0.0)) {
    throw(" Sharpness parameter must be positive! ") ;
  }
  else {

    // Set the number of terminals
    setNumTerms(4) ;

    // Set number of states
    setNumberOfStates(2);

    // create tape
    DenseIntVector var(2,0); // sets var[0] = 0 and var[1] = 0...
    var[1]=1;
    DenseIntVector novar;
    DenseDoubleVector nodelay;
    createTape(var, novar, novar, novar, nodelay) ;
  }
}

void Vctb_cann::eval(adoublev& x,
	       adoublev& vp, adoublev& ip)
{
  vp[0] = x[0] ;	// x[0] = v_in(t)
  vp[1] = x[1] ;        // x[1] = v_out(t)

  ip[0] = zero ;      // Should this be 0?  It's not really 0...	
  ip[1] = -gain*x[0]/pow((1.0 + pow((gain*fabs(x[0])/limit),s)),(1/s)) ;
}
