// This may look like C code, but it is really -*- C++ -*-
//
// This is a Voltage to voltage transducer with tanh behavior.
//
//                 1                        2
//                  o--------+      +---+--o  ---> Iout
//                                  |    
//           	    +  	            |      +
//           	  Vin	           ( )     Vout 
//           	    -	            |      -
//                                  |    
//           	    o--------+      +---+--o
//                 3                        4
//
// by Frank P. Hart

#ifndef Vvtb_tanh_h
#define Vvtb_tanh_h 1

class Vvtb_tanh : public AdolcElement
{
public:
  
  Vvtb_tanh(const string& iname);

  ~Vvtb_tanh() {}

  static const char* getNetlistName()
  {
    return einfo.name;
  }

  // This is a multi-reference element.
  virtual void getLocalRefIdx(UnsignedVector& local_ref_vec,
			      TerminalVector& term_list);
  // fill MNAM
//  virtual void fillMNAM(FreqMNAM* mnam);
//  virtual void fillMNAM(TimeMNAM* mnam);
//  virtual void getExtraRC(unsigned& first_eqn, unsigned& n_rows) const ;
//  virtual unsigned getExtraRC(const unsigned& eqn_number, const MNAMType& type) ;
  virtual void eval(adoublev& x, adoublev& vp, adoublev& ip) ;
  virtual void init() throw(string&) ;
private:

  // Element information
  static ItemInfo einfo;

  // Number of parameters of this element
  static const unsigned n_par;

  // Parameter variables
  double limit;

  // Parameter information
  static ParmInfo pinfo[];

};

#endif
