// Comments:
// 1. Consider moving all memory allocation to interface elements?

#include "AdolcElement3.h"
#include "../analysis/FreqDomainSV.h"
#include "../analysis/TimeDomainSV.h"
#include "../analysis/WaveletDomainSV.h"
#include "adutilsc.h"
#include <cassert>

bool AdolcElement3::SVWSstatus = false;
unsigned AdolcElement3::big_number = 10000;
int AdolcElement3::aux_size = 20;
double* AdolcElement3::x = NULL;
double* AdolcElement3::f = NULL;
double* AdolcElement3::jac_col_sel = NULL;


AdolcElement3::AdolcElement3(ItemInfo* einfo, ParmInfo* param_desc,
const int& numparms, const string& iname)
: Element(einfo, param_desc, numparms, iname),
my_nstates(0), ni_adolc1(0), nd_adolc1(0), ni_adolc2(0), nd_adolc2(0)
{
  // Set to true in createTape()
  taped = false;
}

AdolcElement3::~AdolcElement3()
{
}

// State-variable related methods

void AdolcElement3::allocSVWS()
{
  if (SVWSstatus)
	{
    delete [] jac_col_sel;
    delete [] f;
    delete [] x;
  }
  jac_col_sel = new double[aux_size];
  for (int j=0; j<aux_size; j++)
    jac_col_sel[j] = zero;
  x = new double[aux_size];
  f = new double[aux_size];
  // Set flag
  SVWSstatus = true;
}

void AdolcElement3::createTape1(const DenseIntVector& var,
const DenseIntVector& t_var,
const DenseDoubleVector& delay,
const int ny1, const int nz1)
{
  my_nstates = getNumberOfStates();
  assert(my_nstates);
  // Check sizes
  assert(var.size() <= my_nstates);
  assert(t_var.size() <= my_nstates);
  assert(t_var.size() == delay.size());

	this->var.resize(var.size());
	mtl::copy(var, this->var);
	this->t_var.resize(t_var.size());
	mtl::copy(t_var, this->t_var);
	this->delay.resize(delay.size());
	mtl::copy(delay, this->delay);
  this->ny1 = ny1;
  this->nz1 = nz1;

  // Assign values to var2
  var2.resize(ny1);
  for (int i=0; i<ny1; i++)
    var2[i] = i;

  // set the number of variables used by adol-c
  ni_adolc1 = var.size() + t_var.size();
  nd_adolc1 = ny1 + nz1;
  delconst = var.size();
  Jy1.resize(ny1,my_nstates);
  Jz1.resize(nz1,my_nstates);

  // Find the greater
  int max_dim = nd_adolc1;
  if (ni_adolc1 > nd_adolc1)
    max_dim = ni_adolc1;
  // Reserve memory if needed
  if (!SVWSstatus || (max_dim > aux_size))
	{
    aux_size = max_dim;
    // realloc space
    allocSVWS();
  }

  // Tape the eval() function
  for (int i=0; i<ni_adolc1; i++)
    x[i] = zero;
  // We need some method to store different tapes with the same ID
  assert(getId() < big_number);
  trace_on(getId());
  // inputs
  adoublev ax(var.size()), atx(t_var.size());
  // outputs
  adoublev ay1(ny1), az1(nz1);
  ax <<= x;
  atx <<= x + var.size();
  eval1(ax, atx, ay1, az1);
  ay1 >>= f;
  az1 >>= f + ny1;
  trace_off();
}

void AdolcElement3::reTape1()
{
  //Here we should check if we need re-taping
  trace_on(getId());
  //inputs
  adoublev ax(var.size()), atx(t_var.size());
  //outputs
  adoublev ay1(ny1), az1(nz1);
  ax <<= x;
  atx <<= x + var.size();
  eval1(ax, atx, ay1, az1);
  ay1 >>= f;
  az1 >>= f + ny1;
  trace_off();
}

void AdolcElement3::createTape2(const int ny2, const int nz2)
{
  my_nstates = getNumberOfStates();
  assert(my_nstates);

  this->ny2 = ny2;
  this->nz2 = nz2;
  // Assign values to var3
  var3.resize(ny2);
  for (int i=0; i<ny2; i++)
    var3[i] = ny1 + i;

  Jy2.resize(ny2,max(nz1,ny1));
  Jz2.resize(nz2,max(nz1,ny1));
  Jy2tot.resize(ny2,my_nstates);
  Jz2tot.resize(nz2,my_nstates);

  // set the number of variables used by adol-c
  ni_adolc2 = ny1 + nz1;
  nd_adolc2 = ny2 + nz2;

  // Find the greater
  int max_dim = nd_adolc2;
  if (ni_adolc2 > nd_adolc2)
    max_dim = ni_adolc2;
  // Reserve memory if needed
  if (max_dim > aux_size)
	{
    aux_size = max_dim;
    // realloc space
    allocSVWS();
  }

  // Tape the eval() function
  for (int i=0; i<ni_adolc2; i++)
    x[i] = zero;
  // We need some method to store different tapes with the same ID
  assert(getId() < big_number);
  trace_on(big_number + getId());
  // inputs
  adoublev ady1(ny1), az1(nz1);
  // outputs
  adoublev ay2(ny2), az2(nz2);
  ady1 <<= x;
  az1 <<= x + ny1;
  eval2(ady1, az1, ay2, az2);
  ay2 >>= f;
  az2 >>= f + ny2;
  trace_off();
}

void AdolcElement3::reTape2()
{
  //Here we should check if we need re-taping
  trace_on(getId());
  //inputs
  adoublev ady1(ny1), az1(nz1);
  //outputs
  adoublev ay2(ny2), az2(nz2);
  ady1 <<= x;
  az1 <<= x + ny1;
  eval2(ady1, az1, ay2, az2);
  ay2 >>= f;
  az2 >>= f + ny2;
  trace_off();
}

void AdolcElement3::createTape3()
{
  my_nstates = getNumberOfStates();
  assert(my_nstates);

  // ny3 must necessarily be zero
  // nz3 is also predefined
  this->nz3 = 2 * my_nstates;
  Jz3.resize(nz3,max(nz2,ny2));

  // set the number of variables used by adol-c
  ni_adolc3 = ny2 + nz2;
  nd_adolc3 = nz3;

  // Find the greater
  int max_dim = nd_adolc3;
  if (ni_adolc3 > nd_adolc3)
    max_dim = ni_adolc3;
  // Reserve memory if needed
  if (max_dim > aux_size)
	{
    aux_size = max_dim;
    // realloc space
    allocSVWS();
  }

  // Tape the eval() function
  for (int i=0; i<ni_adolc3; i++)
    x[i] = zero;
  // We need some method to store different tapes with the same ID
  assert(getId() < big_number);
  trace_on(2 * big_number + getId());
  // inputs
  adoublev ady2(ny2), az2(nz2);
  // outputs
  adoublev vp(my_nstates), ip(my_nstates);
  ady2 <<= x;
  az2 <<= x + ny2;
  eval3(ady2, az2, vp, ip);
  vp >>= f;
  ip >>= (f + my_nstates);
  trace_off();
  taped = true;
}

void AdolcElement3::reTape3()
{
  //Here we should check if we need re-taping
  trace_on(getId());
  //inputs
  adoublev ady2(ny2), az2(nz2);
  //outputs
  adoublev vp(my_nstates), ip(my_nstates);
  ady2 <<= x;
  az2 <<= x + ny2;
  eval3(ady2, az2, vp, ip);
  vp >>= f;
  ip >>= (f + my_nstates);
  trace_off();
}

unsigned AdolcElement3::getNumberOfSecStates() const
{
  return ny1+ny2;
}

void AdolcElement3::svTran(TimeDomainSV* tdsv)
{
  assert(taped);

  // Prepare x
  for (unsigned i=0; i<var.size(); i++)
    x[i] = tdsv->getX(var[i]);
  for (unsigned i=0; i<t_var.size(); i++)
    x[delconst + i] = tdsv->getDelayedX(t_var[i], delay[i]);
  if (function(getId(), nd_adolc1, ni_adolc1, x, f) < 0)
	{
		cout << "Tape error" << endl;
		reTape1();
	}
  // retrieve y1 and z1
  // derivate y1 and prepare the new x
  for (int i=0; i<ny1; i++)
	{
    // get derivative
    x[i] = tdsv->getdY_dt(var2[i], f[i]);
  }
  for (int i=0; i<nz1; i++)
    x[ny1+i] = f[ny1+i];
  if (function(getId() + big_number, nd_adolc2, ni_adolc2, x, f) < 0)
	{
		cout << "Tape error" << endl;
		reTape2();
	}
  if (tdsv->DC())
    for (int i=0; i<ny2; i++)
      x[i] = zero;
  else
	{
    // derivate y2 and prepare the new x
    for (int i=0; i<ny2; i++)
		{
      // get derivative
      x[i] = tdsv->getdY_dt(var3[i], f[i]);
    }
  }
  for (int i=0; i<nz2; i++)
    x[ny2+i] = f[ny2+i];
  if (function(getId() + 2 * big_number, nd_adolc3, ni_adolc3, x, f) < 0)
	{
		cout << "Tape error" << endl;
		reTape3();
	}
  // Extract u and i
  for (unsigned i=0; i<my_nstates; i++)
	{
    tdsv->u(i) = f[i];
    tdsv->i(i) = f[my_nstates + i];
  }
  return;
}

void AdolcElement3::deriv_svTran(TimeDomainSV* tdsv)
{
  assert(taped);
  assert(!tdsv->firstTime());

  // Prepare x
  for (unsigned i=0; i<var.size(); i++)
	{
    x[i] = tdsv->getX(var[i]);
  }
  for (unsigned i=0; i<t_var.size(); i++)
	{
    x[delconst + i] = tdsv->getDelayedX(t_var[i], delay[i]);
  }
  // Evaluate derivatives
  tdsv->cleanJac();

  DoubleDenseColMatrix& Ju = tdsv->getJu();
  DoubleDenseColMatrix& Ji = tdsv->getJi();

  // Jacobians for first level
  for (unsigned i=0; i<var.size(); i++)
	{
    jac_col_sel[i] = one;
    if (jac_vec(getId(), nd_adolc1, ni_adolc1, x, jac_col_sel, f) < 0)
		{
      cout << "Tape error" << endl;
      reTape1();
    }
    jac_col_sel[i] = zero;
    for (int j=0; j<ny1; j++)
      Jy1(j,var[i]) = f[j];
    for (int j=0; j<nz1; j++)
      Jz1(j,var[i]) = f[ny1 + j];
  }
  for (unsigned i=0; i<t_var.size(); i++) {
    jac_col_sel[delconst + i] = tdsv->getDelayFactor(delay[i]);
    if (jac_vec(getId(), nd_adolc1, ni_adolc1, x, jac_col_sel, f) < 0)
		{
      cout << "Tape error" << endl;
      reTape1();
    }
    jac_col_sel[delconst + i] = zero;
    for (int j=0; j<ny1; j++)
      Jy1(j,t_var[i]) += f[j];
    for (int j=0; j<nz1; j++)
      Jz1(j,t_var[i]) += f[ny1 + j];
  }

  // Jacobians for third level
  if (function(getId(), nd_adolc1, ni_adolc1, x, f) < 0)
	{
    cout << "Tape error" << endl;
    reTape1();
  }
  // derivate y1 and prepare the new x
  for (int i=0; i<ny2; i++)
	{
    // get derivative
		x[i] = tdsv->getdY_dt(var3[i], f[i]);
	}

  for (int i=0; i<nz1; i++)
    x[ny1+i] = f[ny1+i];
  // Jz2 includes d(dy1/dt)/dx
  for (int i=0; i<ny1; i++)
	{
    // getdx_dtFactor() is the factor used by the i.m.
    // (this might lead to problems) why?
    jac_col_sel[i] = tdsv->getdx_dtFactor();
    if (jac_vec(big_number + getId(), nd_adolc2, ni_adolc2,
			x, jac_col_sel, f) < 0)
		{
      cout << "Tape error" << endl;
      reTape2();
    }
    jac_col_sel[i] = zero;
    for (int j=0; j<ny2; j++)
      Jy2(j,i) = f[j];
    for (int j=0; j<nz2; j++)
      Jz2(j,i) = f[ny2 + j];
  }
  mtl::set_value(Jy2tot, zero);
  mtl::set_value(Jz2tot, zero);

  // Jy2tot += Jy2 * Jy1
  mult(Jy2tot, Jy2, Jy1, ny1);
  // Jz2tot += Jz2 * Jy1
  mult(Jz2tot, Jz2, Jy1, ny1);

  for (int i=0; i<nz1; i++)
	{
    jac_col_sel[ny1 + i] = one;
    if (jac_vec(big_number + getId(), nd_adolc2, ni_adolc2,
			x, jac_col_sel, f) < 0)
		{
      cout << "Tape error" << endl;
      reTape2();
    }
    jac_col_sel[ny1 + i] = zero;
    for (int j=0; j<ny2; j++)
      Jy2(j,i) = f[j];
    for (int j=0; j<nz2; j++)
      Jz2(j,i) = f[ny2 + j];
  }
  // Jy2tot += Jy2 * Jz1
  mult(Jy2tot, Jy2, Jz1, nz1);
  // Jz2tot += Jz2 * Jz1
  mult(Jz2tot, Jz2, Jz1, nz1);

  if (function(big_number + getId(), nd_adolc2, ni_adolc2, x, f) < 0)
	{
		cout << "Tape error" << endl;
		reTape2();
	}
  // derivate y2 and prepare the new x
  for (int i=0; i<ny2; i++)
	{
    // get derivative
		x[i] = tdsv->getdY_dt(var3[i], f[i]);
	}

  for (int i=0; i<nz2; i++)
    x[ny2+i] = f[ny2+i];

  // Jz3 includes d(dy2/dt)/dx
  for (int i=0; i<ny2; i++)
	{
    // getdx_dtFactor() is the factor used by the i.m.
    // (this might lead to problems) why?
    jac_col_sel[i] = tdsv->getdx_dtFactor();
    if (jac_vec(2 * big_number + getId(), nd_adolc3, ni_adolc3,
			x, jac_col_sel, f) < 0)
		{
      cout << "Tape error" << endl;
      reTape3();
    }
    jac_col_sel[i] = zero;
    for (int j=0; j<nz3; j++)
      Jz3(j,i) = f[j];
  }

  // Jz3 * Jy2
  for (unsigned i=0; i<my_nstates; i++)
    for (unsigned j=0; j<my_nstates; j++)
      for (int k=0; k<ny2; k++)
				Ju(i,j) += Jz3(i,k) * Jy2tot(k,j);
  for (unsigned i=0; i<my_nstates; i++)
    for (unsigned j=0; j<my_nstates; j++)
      for (int k=0; k<ny2; k++)
				Ji(i,j) += Jz3(my_nstates + i,k) * Jy2tot(k,j);

  // Second term of Jz3
  for (int i=0; i<nz2; i++)
	{
    jac_col_sel[ny2 + i] = one;
    if (jac_vec(2 * big_number + getId(), nd_adolc3, ni_adolc3,
			x, jac_col_sel, f) < 0)
		{
      cout << "Tape error" << endl;
      reTape3();
    }
    jac_col_sel[ny2 + i] = zero;
    for (int j=0; j<nz3; j++)
      Jz3(j,i) = f[j];
  }

  // Jz3 * Jz2
  for (unsigned i=0; i<my_nstates; i++)
    for (unsigned j=0; j<my_nstates; j++)
      for (int k=0; k<nz2; k++)
				Ju(i,j) += Jz3(i,k) * Jz2tot(k,j);
  for (unsigned i=0; i<my_nstates; i++)
    for (unsigned j=0; j<my_nstates; j++)
      for (int k=0; k<nz2; k++)
				Ji(i,j) += Jz3(my_nstates + i,k) * Jz2tot(k,j);

  return;
}

void AdolcElement3::mult(DoubleDenseColMatrix& c,
DoubleDenseColMatrix& a, DoubleDenseColMatrix& b, const int& kmax)
{
  assert(c.nrows() == a.nrows());
  assert(c.ncols() == b.ncols());
  //  assert(a.size(1) == b.size(0));
  int imax = c.nrows();
  int jmax = c.ncols();
  //  int kmax = a.size(1);
  for (int i=0; i<imax; i++)
    for (int j=0; j<jmax; j++)
      for (int k=0; k<kmax; k++)
				c(i,j) += a(i,k) * b(k,j);
}

//  void AdolcElement3::svHB_timeX(FreqDomainSV*& fdsv)
//  {
	//    // Tell fdsv to prepare the time-domain vectors
	//    for (unsigned i=0; i<var.size(); i++) {
		//      fdsv->transformX(var[i]);
	//    }
	//    for (unsigned i=0; i<t_var.size(); i++) {
		//      fdsv->evalDelX(t_var[i], delay[i]);
		//      fdsv->transformDelX(t_var[i]);
	//    }
//  }

//  void AdolcElement3::svHB_getx(FreqDomainSV*& fdsv, const int& idx)
//  {
	//    // Prepare x
	//    for (unsigned i=0; i<var.size(); i++) {
		//      x[i] = fdsv->getx(var[i])[idx];
	//    }
	//    for (unsigned i=0; i<t_var.size(); i++) {
		//      x[delconst + i] = fdsv->getDelx(t_var[i])[idx];
	//    }
//  }

//  void AdolcElement3::svHB(FreqDomainSV* fdsv)
//  {
	//    assert(taped);
	//    const int& noSamples = fdsv->noSamples();

	//    // Prepare time-domain inputs
	//    svHB_timeX(fdsv);

	//    // Loop through all time samples
	//    for (int idx=0; idx < noSamples ; idx++) {
		//      // prepare x
		//      svHB_getx(fdsv, idx);

		//      if(function(getId(), nd_adolc, ni_adolc, x, f) < 0)
		//        reTape();
		//      // store output
		//      for (unsigned i=0; i<my_nstates; i++) {
			//        fdsv->vp(i)[idx] = f[i];
			//        fdsv->ip(i)[idx] = f[my_nstates + i];
		//      }
	//    }
	//    // Tell fdsv to convert the output back to the freq. domain
	//    for (unsigned i=0; i<my_nstates; i++) {
		//      fdsv->transformvp(i);
		//      fdsv->transformip(i);
	//    }
//  }

