#include "ElementData.h"

ElementData::ElementData(const unsigned& numterms)
{
  assert(numterms);
  this->numterms = numterms;

  // Allocate number of blank output vectors
  it_vv = new DenseDoubleVector[numterms];
  ut_vv = new DenseDoubleVector[numterms];
  xt_vv = new DenseDoubleVector[numterms];

  if_vv = new DenseComplexVector[numterms];
  xf_vv = new DenseComplexVector[numterms];
}

ElementData::~ElementData()
{
  // This memory is allocated when numterms is set.
  delete [] it_vv;
  delete [] ut_vv;
  delete [] xt_vv;

  delete [] if_vv;
  delete [] xf_vv;
}

void ElementData::setRealI(const unsigned& index, const DenseDoubleVector& i_v)
{
  assert(index < numterms);
	this->it_vv[index].resize(i_v.size());
	mtl::copy(i_v, this->it_vv[index]);
}

void ElementData::setRealI(const unsigned& index, const double i_v)
{
  this->it_vv[index].push_back(i_v);
}

const DenseDoubleVector& ElementData::getRealI(const unsigned& index) const
{
  assert(index < numterms);
  return it_vv[index];
}

void ElementData::setRealU(const unsigned& index, const DenseDoubleVector& u_v)
{
  assert(index < numterms);
	this->ut_vv[index].resize(u_v.size());
	mtl::copy(u_v, this->ut_vv[index]);
}

void ElementData::setRealU(const unsigned& index, const double u_v)
{
  this->ut_vv[index].push_back(u_v);
}

const DenseDoubleVector& ElementData::getRealU(const unsigned& index) const
{
  assert(index < numterms);
  return ut_vv[index];
}

void ElementData::setRealX(const unsigned& index, const DenseDoubleVector& x_v)
{
  assert(index < numterms);
  this->xt_vv[index].resize(x_v.size());
	mtl::copy(x_v, this->xt_vv[index]);
}

void ElementData::setRealX(const unsigned& index, const double x_v)
{
  this->xt_vv[index].push_back(x_v);
}

const DenseDoubleVector& ElementData::getRealX(const unsigned& index) const
{
  assert(index < numterms);
  return xt_vv[index];
}

void ElementData::setPhasorI(const unsigned& index, const DenseComplexVector& i_v)
{
  assert(index < numterms);
	this->if_vv[index].resize(i_v.size());
	mtl::copy(i_v, this->if_vv[index]);
}

const DenseComplexVector& ElementData::getPhasorI(const unsigned& index) const
{
  assert(index < numterms);
  return if_vv[index];
}

void ElementData::setPhasorX(const unsigned& index, const DenseComplexVector& x_v)
{
  assert(index < numterms);
	this->xf_vv[index].resize(x_v.size());
	mtl::copy(x_v, this->xf_vv[index]);
}

const DenseComplexVector& ElementData::getPhasorX(const unsigned& index) const
{
  assert(index < numterms);
  return xf_vv[index];
}

